@@ -1,27 +1,13 @@
+/*
+ * SPDX-License-Identifier: MIT
+ *
+ * Copyright (C) 2010-2019 EfficiOS Inc. and Linux Foundation
+ */
+
 #ifndef BABELTRACE2_GRAPH_GRAPH_H
 #define BABELTRACE2_GRAPH_GRAPH_H
 
-/*
- * Copyright (c) 2010-2019 EfficiOS Inc. and Linux Foundation
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- */
+/* IWYU pragma: private, include <babeltrace2/babeltrace.h> */
 
 #ifndef __BT_IN_BABELTRACE_H
 # error "Please include <babeltrace2/babeltrace.h> instead."
@@ -71,7 +57,7 @@ In the illustrations above, notice that:
   writes to the standard output or to a file.
 
 - There are two source components having the same
-  \ref api-tir-comp-cls "class": <code>source.ctf.fs</code>.
+  \ref api-comp-cls "class": <code>source.ctf.fs</code>.
 
 - All components are instances of \bt_plugin-provided classes:
   <code>babeltrace2-plugin-ctf.so</code>,
@@ -91,12 +77,12 @@ dynamically (see \ref api-comp-cls-dev).
 The connections between component ports in a trace processing graph
 indicate the needed topology to achieve a trace processing task. That
 being said,
-\bt_p_msg do not flow within connections. Instead, a source-to-sink
+\bt_p_msg don't flow within connections. Instead, a source-to-sink
 or filter-to-sink connection makes it
 possible for a sink component to create a \bt_msg_iter on its end of
 the connection (an \bt_iport). In turn, a filter component message
-iterator can create other message iterators on one or more of the
-component's input ports. For a single connection, there can exist
+iterator can create other message iterators on one or more input ports
+of the component. For a single connection, there can exist
 multiple message iterators.
 
 A trace processing graph is a
@@ -116,7 +102,7 @@ run it.
 To interrupt a \ref api-graph-lc-run "running" trace processing graph,
 either:
 
-- Borrow the graph's default \bt_intr with
+- Borrow the default \bt_intr of the graph with
   bt_graph_borrow_default_interrupter() and set it with
   bt_interrupter_set().
 
@@ -171,9 +157,9 @@ In general, as of \bt_name_version_min_maj:
 - Components cannot remove ports.
 
 If \em any error occurs (a function returns a status code which ends
-with <code>ERROR</code>) during step 1 or 2, the trace processing
-graph is considered <strong>faulty</strong>: you cannot use it anymore.
-The only functions you can call with a faulty graph are
+with <code>ERROR</code>) during step&nbsp;1 or&nbsp;2, then the trace
+processing graph is considered <strong>faulty</strong>: you cannot use
+it anymore. The only functions you can call with a faulty graph are
 bt_graph_get_ref() and bt_graph_put_ref().
 
 <h2>\anchor api-graph-lc-add Add components</h2>
@@ -222,8 +208,10 @@ to add a component to a trace processing graph:
 </dl>
 
 The <code>*_with_initialize_method_data()</code> versions can pass a
-custom, \bt_voidp pointer to the component's
-\ref api-comp-cls-dev-meth-init "initialization method".
+custom, \bt_voidp pointer to the
+\ref api-comp-cls-dev-meth-init "initialization method" of the
+component.
+
 The other versions pass \c NULL as this parameter.
 
 All the functions above accept the same parameters:
@@ -269,11 +257,12 @@ this function, you pass three user functions:
 
   <dt>Consume</dt>
   <dd>
-    Called every time the sink component's
-    \ref api-comp-cls-dev-meth-consume "consuming method" is called.
+    Called every time the
+    \ref api-comp-cls-dev-meth-consume "consuming method" of the
+    sink component is called.
 
-    You can get the next messages from the component's message
-    iterator and process them.
+    You can get the next messages from the message
+    iterator of the component and process them.
   </dd>
 
   <dt>\bt_dt_opt Finalization</dt>
@@ -313,7 +302,7 @@ when you \ref api-graph-lc-run "run" the trace processing graph:
 
 <h2>\anchor api-graph-lc-run Run</h2>
 
-When you are done configuring a trace processing graph, you can run it
+When you're done configuring a trace processing graph, you can run it
 with bt_graph_run().
 
 bt_graph_run() does \em not return until one of:
@@ -333,13 +322,13 @@ bt_graph_run() does \em not return until one of:
 
   This feature exists to allow blocking operations within components
   to be postponed until they don't block. The graph user can perform
-  other tasks instead of the graph's thread blocking.
+  other tasks instead of blocking the thread of the graph.
 
 - The trace processing graph is interrupted (see
   bt_graph_borrow_default_interrupter() and bt_graph_add_interrupter()):
   bt_graph_run() returns #BT_GRAPH_RUN_STATUS_AGAIN.
 
-  Check the \bt_intr's state with bt_interrupter_is_set() to
+  Check the state of the \bt_intr with bt_interrupter_is_set() to
   distinguish between a sink component returning "try again" and
   the trace processing graph getting interrupted.
 
@@ -429,7 +418,7 @@ graph topologies. They are:
         to a file.
 
         This is equivalent to the \c text output format of the
-        <a href="http://man7.org/linux/man-pages/man1/babeltrace.1.html">Babeltrace&nbsp;1
+        <a href="https://manpages.debian.org/stretch/babeltrace/babeltrace.1.en.html">Babeltrace&nbsp;1
         command-line tool</a>.
 
         See \bt_man{babeltrace2-sink.text.pretty,7}.
@@ -556,13 +545,11 @@ The graph is configured as operating following version
 bt_self_component_get_graph_mip_version() returns this version when
 components call it.
 
-@note
-    As of \bt_name_version_min_maj, the only available MIP version is 0.
-
 The returned graph has a default \bt_intr. Any \bt_comp you add with the
 <code>bt_graph_add_*_component*()</code> functions and all their
-\bt_p_msg_iter also have this same default interrupter. Borrow the graph's
-default interrupter with bt_graph_borrow_default_interrupter().
+\bt_p_msg_iter also have this same default interrupter. Borrow the
+default interrupter of the graph
+with bt_graph_borrow_default_interrupter().
 
 @param[in] mip_version
     Version of the Message Interchange Protocol to use within the
@@ -572,9 +559,9 @@ default interrupter with bt_graph_borrow_default_interrupter().
     New trace processing graph reference, or \c NULL on memory error.
 
 @pre
-    \bt_p{mip_version} is 0.
+    \bt_p{mip_version} is ≤ what bt_get_maximal_mip_version() returns.
 */
-extern bt_graph *bt_graph_create(uint64_t mip_version);
+extern bt_graph *bt_graph_create(uint64_t mip_version) __BT_NOEXCEPT;
 
 /*! @} */
 
@@ -618,12 +605,12 @@ bt_graph_add_source_component(bt_graph *graph,
 		const bt_component_class_source *component_class,
 		const char *name, const bt_value *params,
 		bt_logging_level logging_level,
-		const bt_component_source **component);
+		const bt_component_source **component) __BT_NOEXCEPT;
 
 /*!
 @brief
     Creates a \bt_src_comp from the
-    \ref api-tir-comp-cls "class" \bt_p{component_class}
+    \ref api-comp-cls "class" \bt_p{component_class}
     with the initialization parameters \bt_p{params}, the initialization
     user data \bt_p{initialize_method_data}, and the initial
     logging level \bt_p{logging_level}, adds it to the trace processing
@@ -633,11 +620,11 @@ bt_graph_add_source_component(bt_graph *graph,
 See \ref api-graph-lc-add "Add components" to learn more about adding
 components to a trace processing graph.
 
-This function calls the source component's
-\ref api-comp-cls-dev-meth-init "initialization method" after
-creating it.
+This function calls the
+\ref api-comp-cls-dev-meth-init "initialization method" of
+the source component after creating it.
 
-The created source component's initialization method receives:
+The initialization method of the created source component receives:
 
 - \bt_p{params} as its own \bt_p{params} parameter (or an empty
   \bt_map_val if \bt_p{params} is \c NULL).
@@ -658,13 +645,13 @@ The created source component can get its logging level
     @parblock
     Initialization parameters to use when creating the source component.
 
-    Can be \c NULL, in which case the created source component's
-    initialization method receives an empty \bt_map_val as its
+    Can be \c NULL, in which case the initialization method of the
+    created source component receives an empty \bt_map_val as its
     \bt_p{params} parameter.
     @endparblock
 @param[in] initialize_method_data
-    User data passed as is to the created source component's
-    initialization method.
+    User data passed as is to the initialization method of the
+    created source component.
 @param[in] logging_level
     Initial logging level of the source component to create.
 @param[out] component
@@ -676,8 +663,8 @@ The created source component can get its logging level
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_MEMORY_ERROR
     Out of memory.
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_ERROR
-    Other error, for example, the created source component's
-    initialization method failed.
+    Other error, for example, the initialization method of the
+    created source component failed.
 
 @bt_pre_not_null{graph}
 @bt_pre_graph_not_configured{graph}
@@ -689,6 +676,13 @@ The created source component can get its logging level
 @pre
     \bt_p{params} is a \bt_map_val (bt_value_is_map() returns #BT_TRUE)
     or is \c NULL.
+@pre
+    The
+    \link api-comp-cls-dev-meth-mip "get supported MIP versions"\endlink
+    method of \bt_p{component_class}, given the same \bt_p{params},
+    \bt_p{initialize_method_data}, and \bt_p{logging_level} parameters,
+    makes one of its supported \bt_mip (MIP) version ranges contain the
+    effective MIP version of \bt_p{graph} (see bt_graph_create()).
 
 @bt_post_success_frozen{component_class}
 @bt_post_success_frozen{params}
@@ -699,7 +693,7 @@ bt_graph_add_source_component_with_initialize_method_data(
 		const bt_component_class_source *component_class,
 		const char *name, const bt_value *params,
 		void *initialize_method_data, bt_logging_level logging_level,
-		const bt_component_source **component);
+		const bt_component_source **component) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -711,12 +705,12 @@ bt_graph_add_filter_component(bt_graph *graph,
 		const bt_component_class_filter *component_class,
 		const char *name, const bt_value *params,
 		bt_logging_level logging_level,
-		const bt_component_filter **component);
+		const bt_component_filter **component) __BT_NOEXCEPT;
 
 /*!
 @brief
     Creates a \bt_flt_comp from the
-    \ref api-tir-comp-cls "class" \bt_p{component_class}
+    \ref api-comp-cls "class" \bt_p{component_class}
     with the initialization parameters \bt_p{params}, the initialization
     user data \bt_p{initialize_method_data}, and the initial
     logging level \bt_p{logging_level}, adds it to the trace processing
@@ -726,11 +720,11 @@ bt_graph_add_filter_component(bt_graph *graph,
 See \ref api-graph-lc-add "Add components" to learn more about adding
 components to a trace processing graph.
 
-This function calls the filter component's
-\ref api-comp-cls-dev-meth-init "initialization method" after
-creating it.
+This function calls the
+\ref api-comp-cls-dev-meth-init "initialization method" of the
+filter component after creating it.
 
-The created filter component's initialization method receives:
+The initialization method of the created filter component receives:
 
 - \bt_p{params} as its own \bt_p{params} parameter (or an empty
   \bt_map_val if \bt_p{params} is \c NULL).
@@ -751,13 +745,13 @@ The created filter component can get its logging level
     @parblock
     Initialization parameters to use when creating the filter component.
 
-    Can be \c NULL, in which case the created filter component's
-    initialization method receives an empty \bt_map_val as its
+    Can be \c NULL, in which case the initialization method of the
+    created filter component receives an empty \bt_map_val as its
     \bt_p{params} parameter.
     @endparblock
 @param[in] initialize_method_data
-    User data passed as is to the created filter component's
-    initialization method.
+    User data passed as is to the initialization method of the
+    created filter component.
 @param[in] logging_level
     Initial logging level of the filter component to create.
 @param[out] component
@@ -769,8 +763,8 @@ The created filter component can get its logging level
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_MEMORY_ERROR
     Out of memory.
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_ERROR
-    Other error, for example, the created filter component's
-    initialization method failed.
+    Other error, for example, the initialization method of the
+    created filter component failed.
 
 @bt_pre_not_null{graph}
 @bt_pre_graph_not_configured{graph}
@@ -782,6 +776,13 @@ The created filter component can get its logging level
 @pre
     \bt_p{params} is a \bt_map_val (bt_value_is_map() returns #BT_TRUE)
     or is \c NULL.
+@pre
+    The
+    \link api-comp-cls-dev-meth-mip "get supported MIP versions"\endlink
+    method of \bt_p{component_class}, given the same \bt_p{params},
+    \bt_p{initialize_method_data}, and \bt_p{logging_level} parameters,
+    makes one of its supported \bt_mip (MIP) version ranges contain the
+    effective MIP version of \bt_p{graph} (see bt_graph_create()).
 
 @bt_post_success_frozen{component_class}
 @bt_post_success_frozen{params}
@@ -792,7 +793,7 @@ bt_graph_add_filter_component_with_initialize_method_data(
 		const bt_component_class_filter *component_class,
 		const char *name, const bt_value *params,
 		void *initialize_method_data, bt_logging_level logging_level,
-		const bt_component_filter **component);
+		const bt_component_filter **component) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -804,12 +805,12 @@ bt_graph_add_sink_component(
 		bt_graph *graph, const bt_component_class_sink *component_class,
 		const char *name, const bt_value *params,
 		bt_logging_level logging_level,
-		const bt_component_sink **component);
+		const bt_component_sink **component) __BT_NOEXCEPT;
 
 /*!
 @brief
     Creates a \bt_sink_comp from the
-    \ref api-tir-comp-cls "class" \bt_p{component_class}
+    \ref api-comp-cls "class" \bt_p{component_class}
     with the initialization parameters \bt_p{params}, the initialization
     user data \bt_p{initialize_method_data}, and the initial
     logging level \bt_p{logging_level}, adds it to the trace processing
@@ -819,11 +820,11 @@ bt_graph_add_sink_component(
 See \ref api-graph-lc-add "Add components" to learn more about adding
 components to a trace processing graph.
 
-This function calls the sink component's
-\ref api-comp-cls-dev-meth-init "initialization method" after
-creating it.
+This function calls the
+\ref api-comp-cls-dev-meth-init "initialization method"
+of the sink component after creating it.
 
-The created sink component's initialization method receives:
+The initialization method of the created sink component receives:
 
 - \bt_p{params} as its own \bt_p{params} parameter (or an empty
   \bt_map_val if \bt_p{params} is \c NULL).
@@ -844,13 +845,13 @@ The created sink component can get its logging level
     @parblock
     Initialization parameters to use when creating the sink component.
 
-    Can be \c NULL, in which case the created sink component's
-    initialization method receives an empty \bt_map_val as its
+    Can be \c NULL, in which case the initialization method of the
+    created sink component receives an empty \bt_map_val as its
     \bt_p{params} parameter.
     @endparblock
 @param[in] initialize_method_data
-    User data passed as is to the created sink component's
-    initialization method.
+    User data passed as is to the initialization method of the
+    created sink component.
 @param[in] logging_level
     Initial logging level of the sink component to create.
 @param[out] component
@@ -862,8 +863,8 @@ The created sink component can get its logging level
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_MEMORY_ERROR
     Out of memory.
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_ERROR
-    Other error, for example, the created sink component's
-    initialization method failed.
+    Other error, for example, the initialization method of the
+    created sink component failed.
 
 @bt_pre_not_null{graph}
 @bt_pre_graph_not_configured{graph}
@@ -875,6 +876,13 @@ The created sink component can get its logging level
 @pre
     \bt_p{params} is a \bt_map_val (bt_value_is_map() returns #BT_TRUE)
     or is \c NULL.
+@pre
+    The
+    \link api-comp-cls-dev-meth-mip "get supported MIP versions"\endlink
+    method of \bt_p{component_class}, given the same \bt_p{params},
+    \bt_p{initialize_method_data}, and \bt_p{logging_level} parameters,
+    makes one of its supported \bt_mip (MIP) version ranges contain the
+    effective MIP version of \bt_p{graph} (see bt_graph_create()).
 
 @bt_post_success_frozen{component_class}
 @bt_post_success_frozen{params}
@@ -884,7 +892,7 @@ bt_graph_add_sink_component_with_initialize_method_data(
 		bt_graph *graph, const bt_component_class_sink *component_class,
 		const char *name, const bt_value *params,
 		void *initialize_method_data, bt_logging_level logging_level,
-		const bt_component_sink **component);
+		const bt_component_sink **component) __BT_NOEXCEPT;
 
 /*! @} */
 
@@ -933,10 +941,10 @@ about adding a simple component to a trace processing graph.
 
 @param[in] message_iterator
     @parblock
-    Simple sink component's upstream message iterator.
+    Upstream message iterator of the simple sink component.
 
-    This user function is free to get the message iterator's next
-    message or to make it seek.
+    This user function is free to get the next
+    message of the message iterator or to make it seek.
     @endparblock
 @param[in] user_data
     User data, as passed as the \bt_p{user_data} parameter of
@@ -952,7 +960,7 @@ about adding a simple component to a trace processing graph.
 @bt_pre_not_null{message_iterator}
 
 @post
-    The reference count of \bt_p{message_iterator} is not changed.
+    The reference count of \bt_p{message_iterator} isn't changed.
 
 @sa bt_graph_add_simple_sink_component() &mdash;
     Creates and adds a simple sink component to a trace processing
@@ -1004,31 +1012,30 @@ typedef enum bt_graph_simple_sink_component_consume_func_status {
 @brief
     User consuming function for bt_graph_add_simple_sink_component().
 
-Such a consuming function is called when the simple sink component's own
-\ref api-comp-cls-dev-meth-consume "consuming method" is called. This
-occurs in a loop within bt_graph_run() or when it's this sink
-component's turn to consume in
-bt_graph_run_once().
+Such a consuming function is called when the own
+\ref api-comp-cls-dev-meth-consume "consuming method" of the simple sink
+component is called. This occurs in a loop within bt_graph_run() or when
+it's the turn of this sink component to consume in bt_graph_run_once().
 
 See \ref api-graph-lc-add-ss "Add a simple sink component" to learn more
 about adding a simple component to a trace processing graph.
 
-If you are not done consuming messages, return
+If you're not done consuming messages, then return
 #BT_GRAPH_SIMPLE_SINK_COMPONENT_CONSUME_FUNC_STATUS_OK.
 
-If you are done consuming messages, return
+If you're done consuming messages, then return
 #BT_GRAPH_SIMPLE_SINK_COMPONENT_CONSUME_FUNC_STATUS_END.
 
 If you wish to avoid a blocking operation and make
-bt_graph_run() or bt_graph_run_once() aware, return
+bt_graph_run() or bt_graph_run_once() aware, then return
 #BT_GRAPH_SIMPLE_SINK_COMPONENT_CONSUME_FUNC_STATUS_AGAIN.
 
 @param[in] message_iterator
     @parblock
-    Simple sink component's upstream message iterator.
+    Upstream message iterator of the simple sink component.
 
-    This user function is free to get the message iterator's next
-    message or to make it seek.
+    This user function is free to get the next message of the message
+    iterator or to make it seek.
     @endparblock
 @param[in] user_data
     User data, as passed as the \bt_p{user_data} parameter of
@@ -1048,7 +1055,7 @@ bt_graph_run() or bt_graph_run_once() aware, return
 @bt_pre_not_null{message_iterator}
 
 @post
-    The reference count of \bt_p{message_iterator} is not changed.
+    The reference count of \bt_p{message_iterator} isn't changed.
 
 @sa bt_graph_add_simple_sink_component() &mdash;
     Creates and adds a simple sink component to a trace processing
@@ -1128,8 +1135,9 @@ as their own \bt_p{user_data} parameter.
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_MEMORY_ERROR
     Out of memory.
 @retval #BT_GRAPH_ADD_COMPONENT_STATUS_ERROR
-    Other error, for example, the created sink component's
-    \ref api-comp-cls-dev-meth-init "initialization method" failed.
+    Other error, for example, the
+    \ref api-comp-cls-dev-meth-init "initialization method" of the
+    created sink component failed.
 
 @bt_pre_not_null{graph}
 @bt_pre_graph_not_configured{graph}
@@ -1144,7 +1152,7 @@ bt_graph_add_simple_sink_component(bt_graph *graph, const char *name,
 		bt_graph_simple_sink_component_initialize_func initialize_func,
 		bt_graph_simple_sink_component_consume_func consume_func,
 		bt_graph_simple_sink_component_finalize_func finalize_func,
-		void *user_data, const bt_component_sink **component);
+		void *user_data, const bt_component_sink **component) __BT_NOEXCEPT;
 
 /*! @} */
 
@@ -1227,13 +1235,13 @@ this function.
     \bt_p{downstream_port} is unconnected
     (bt_port_is_connected() returns #BT_FALSE).
 @pre
-    Connecting \bt_p{upstream_port} to \bt_p{downstream_port} does not
+    Connecting \bt_p{upstream_port} to \bt_p{downstream_port} doesn't
     form a connection cycle within \bt_p{graph}.
 */
 extern bt_graph_connect_ports_status bt_graph_connect_ports(bt_graph *graph,
 		const bt_port_output *upstream_port,
 		const bt_port_input *downstream_port,
-		const bt_connection **connection);
+		const bt_connection **connection) __BT_NOEXCEPT;
 
 /*! @} */
 
@@ -1274,11 +1282,10 @@ typedef enum bt_graph_run_status {
 
 /*!
 @brief
-    Runs the trace processing graph \bt_p{graph}, calling each
-    \bt_sink_comp's
-    \ref api-comp-cls-dev-meth-consume "consuming method" in a round
-    robin fashion until they are all done consuming or an exception
-    occurs.
+    Runs the trace processing graph \bt_p{graph}, calling the
+    \ref api-comp-cls-dev-meth-consume "consuming method" of each
+    \bt_sink_comp in a round robin fashion until they're all done
+    consuming or an exception occurs.
 
 See \ref api-graph-lc-run "Run" to learn more about running a trace
 processing graph.
@@ -1300,18 +1307,18 @@ This function does \em not return until one of:
 
   This feature exists to allow blocking operations within components
   to be postponed until they don't block. The graph user can perform
-  other tasks instead of the graph's thread blocking.
+  other tasks instead of blocking the thread of the graph.
 
 - \bt_p{graph} is interrupted (see bt_graph_borrow_default_interrupter()
   and bt_graph_add_interrupter()), in which case this function returns
   #BT_GRAPH_RUN_STATUS_AGAIN.
 
-  Check the \bt_intr's state with bt_interrupter_is_set() to
+  Check the state of the \bt_intr with bt_interrupter_is_set() to
   distinguish between a sink component returning "try again" and
   \bt_p{graph} getting interrupted.
 
-To make a single sink component consume, then get the thread's control
-back, use bt_graph_run_once().
+To make a single sink component consume, then get the control of the
+thread back, use bt_graph_run_once().
 
 When you call this function or bt_graph_run_once() for the first time,
 \bt_p{graph} becomes <em>configured</em>. See
@@ -1346,10 +1353,10 @@ and what you can and cannot do with a configured graph.
     \bt_p{graph} contains at least one sink component.
 
 @sa bt_graph_run_once() &mdash;
-    Calls a single trace processing graph's sink component's consuming
-    method once.
+    Calls the consuming method of a single sink component of a trace
+    processing graph once.
 */
-extern bt_graph_run_status bt_graph_run(bt_graph *graph);
+extern bt_graph_run_status bt_graph_run(bt_graph *graph) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -1407,7 +1414,7 @@ example, if \bt_p{graph} has two non-ended sink components A and B:
 Considering this, if \bt_p{graph} contains a single non-ended sink
 component, this function \em always makes this sink component consume.
 
-If the sink component's consuming method:
+If the consuming method of the sink component:
 
 <dl>
   <dt>Succeeds</dt>
@@ -1452,12 +1459,23 @@ and what you can and cannot do with a configured graph.
 
 @bt_pre_not_null{graph}
 @bt_pre_graph_not_faulty{graph}
+@pre
+    For each \bt_src_comp within \bt_p{graph}, at least one \bt_oport
+    is connected.
+@pre
+    For each \bt_flt_comp within \bt_p{graph}, at least one \bt_iport
+    and one \bt_iport are connected.
+@pre
+    For each \bt_sink_comp within \bt_p{graph}, at least one \bt_iport
+    is connected.
+@pre
+    \bt_p{graph} contains at least one sink component.
 
 @sa bt_graph_run() &mdash;
     Runs a trace processing graph, making all its sink components
     consume in a round robin fashion.
 */
-extern bt_graph_run_once_status bt_graph_run_once(bt_graph *graph);
+extern bt_graph_run_once_status bt_graph_run_once(bt_graph *graph) __BT_NOEXCEPT;
 
 /*! @} */
 
@@ -1490,14 +1508,14 @@ typedef enum bt_graph_add_interrupter_status {
     \bt_p_sink_comp and \bt_p_msg_iter of the trace processing graph
     \bt_p{graph}, as well as to the graph itself.
 
-Sink components can check whether or not they are interrupted
+Sink components can check whether or not they're interrupted
 with bt_self_component_sink_is_interrupted().
 
-Message iterators can check whether or not they are interrupted
+Message iterators can check whether or not they're interrupted
 with bt_self_message_iterator_is_interrupted().
 
 The bt_graph_run() loop intermittently checks whether or not any of the
-graph's interrupters is set. If so, bt_graph_run() returns
+interrupters of the graph is set. If so, then bt_graph_run() returns
 #BT_GRAPH_RUN_STATUS_AGAIN.
 
 @note
@@ -1531,7 +1549,7 @@ graph's interrupters is set. If so, bt_graph_run() returns
     Borrows the default interrupter from a trace processing graph.
 */
 extern bt_graph_add_interrupter_status bt_graph_add_interrupter(bt_graph *graph,
-		const bt_interrupter *interrupter);
+		const bt_interrupter *interrupter) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -1553,7 +1571,8 @@ extern bt_graph_add_interrupter_status bt_graph_add_interrupter(bt_graph *graph,
 @sa bt_graph_add_interrupter() &mdash;
     Adds an interrupter to a trace processing graph.
 */
-extern bt_interrupter *bt_graph_borrow_default_interrupter(bt_graph *graph);
+extern bt_interrupter *bt_graph_borrow_default_interrupter(bt_graph *graph)
+		__BT_NOEXCEPT;
 
 /*! @} */
 
@@ -1682,7 +1701,7 @@ extern bt_graph_add_listener_status
 bt_graph_add_filter_component_input_port_added_listener(
 		bt_graph *graph,
 		bt_graph_filter_component_input_port_added_listener_func user_func,
-		void *user_data, bt_listener_id *listener_id);
+		void *user_data, bt_listener_id *listener_id) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -1758,7 +1777,7 @@ extern bt_graph_add_listener_status
 bt_graph_add_sink_component_input_port_added_listener(
 		bt_graph *graph,
 		bt_graph_sink_component_input_port_added_listener_func user_func,
-		void *user_data, bt_listener_id *listener_id);
+		void *user_data, bt_listener_id *listener_id) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -1834,7 +1853,7 @@ extern bt_graph_add_listener_status
 bt_graph_add_source_component_output_port_added_listener(
 		bt_graph *graph,
 		bt_graph_source_component_output_port_added_listener_func user_func,
-		void *user_data, bt_listener_id *listener_id);
+		void *user_data, bt_listener_id *listener_id) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -1910,7 +1929,7 @@ extern bt_graph_add_listener_status
 bt_graph_add_filter_component_output_port_added_listener(
 		bt_graph *graph,
 		bt_graph_filter_component_output_port_added_listener_func user_func,
-		void *user_data, bt_listener_id *listener_id);
+		void *user_data, bt_listener_id *listener_id) __BT_NOEXCEPT;
 
 /*! @} */
 
@@ -1934,7 +1953,7 @@ bt_graph_add_filter_component_output_port_added_listener(
 @sa bt_graph_put_ref() &mdash;
     Decrements the reference count of a trace processing graph.
 */
-extern void bt_graph_get_ref(const bt_graph *graph);
+extern void bt_graph_get_ref(const bt_graph *graph) __BT_NOEXCEPT;
 
 /*!
 @brief
@@ -1951,7 +1970,7 @@ extern void bt_graph_get_ref(const bt_graph *graph);
 @sa bt_graph_get_ref() &mdash;
     Increments the reference count of a trace processing graph.
 */
-extern void bt_graph_put_ref(const bt_graph *graph);
+extern void bt_graph_put_ref(const bt_graph *graph) __BT_NOEXCEPT;
 
 /*!
 @brief
