@@ -1,24 +1,8 @@
 /*
+ * SPDX-License-Identifier: MIT
+ *
  * Copyright 2017-2018 Philippe Proulx <pproulx@efficios.com>
  * Copyright 2015 Jérémie Galarneau <jeremie.galarneau@efficios.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
  */
 
 #define BT_LOG_TAG "LIB/COMPONENT"
@@ -26,8 +10,7 @@
 
 #include "common/common.h"
 #include "common/assert.h"
-#include "lib/assert-pre.h"
-#include "lib/assert-post.h"
+#include "lib/assert-cond.h"
 #include <babeltrace2/graph/self-component.h>
 #include <babeltrace2/graph/component.h>
 #include <babeltrace2/graph/graph.h>
@@ -35,9 +18,7 @@
 #include "compat/compiler.h"
 #include <babeltrace2/types.h>
 #include <babeltrace2/value.h>
-#include "lib/value.h"
 #include <stdint.h>
-#include <inttypes.h>
 
 #include "component.h"
 #include "component-class.h"
@@ -46,29 +27,22 @@
 #include "component-sink.h"
 #include "connection.h"
 #include "graph.h"
-#include "message/iterator.h"
+#include "iterator.h"
 #include "port.h"
 #include "lib/func-status.h"
 
 static
-struct bt_component * (* const component_create_funcs[])(
-		const struct bt_component_class *) = {
+struct bt_component * (* const component_create_funcs[])(void) = {
 	[BT_COMPONENT_CLASS_TYPE_SOURCE] = bt_component_source_create,
 	[BT_COMPONENT_CLASS_TYPE_SINK] = bt_component_sink_create,
 	[BT_COMPONENT_CLASS_TYPE_FILTER] = bt_component_filter_create,
 };
 
-static
-void (*component_destroy_funcs[])(struct bt_component *) = {
-	[BT_COMPONENT_CLASS_TYPE_SOURCE] = bt_component_source_destroy,
-	[BT_COMPONENT_CLASS_TYPE_SINK] = bt_component_sink_destroy,
-	[BT_COMPONENT_CLASS_TYPE_FILTER] = bt_component_filter_destroy,
-};
-
 static
 void finalize_component(struct bt_component *comp)
 {
 	typedef void (*method_t)(void *);
+	const char *method_name;
 
 	method_t method = NULL;
 
@@ -80,6 +54,7 @@ void finalize_component(struct bt_component *comp)
 		struct bt_component_class_source *src_cc = (void *) comp->class;
 
 		method = (method_t) src_cc->methods.finalize;
+		method_name = "bt_component_class_source_finalize_method";
 		break;
 	}
 	case BT_COMPONENT_CLASS_TYPE_FILTER:
@@ -87,6 +62,7 @@ void finalize_component(struct bt_component *comp)
 		struct bt_component_class_filter *flt_cc = (void *) comp->class;
 
 		method = (method_t) flt_cc->methods.finalize;
+		method_name = "bt_component_class_filter_finalize_method";
 		break;
 	}
 	case BT_COMPONENT_CLASS_TYPE_SINK:
@@ -94,6 +70,7 @@ void finalize_component(struct bt_component *comp)
 		struct bt_component_class_sink *sink_cc = (void *) comp->class;
 
 		method = (method_t) sink_cc->methods.finalize;
+		method_name = "bt_component_class_sink_finalize_method";
 		break;
 	}
 	default:
@@ -108,7 +85,7 @@ void finalize_component(struct bt_component *comp)
 		BT_LIB_LOGI("Calling user's component finalization method: "
 			"%![comp-]+c", comp);
 		method(comp);
-		BT_ASSERT_POST_NO_ERROR();
+		BT_ASSERT_POST_NO_ERROR(method_name);
 
 		if (saved_error) {
 			BT_CURRENT_THREAD_MOVE_ERROR_AND_RESET(saved_error);
@@ -146,7 +123,7 @@ void destroy_component(struct bt_object *obj)
 
 	for (i = component->destroy_listeners->len - 1; i >= 0; i--) {
 		struct bt_component_destroy_listener *listener =
-			&g_array_index(component->destroy_listeners,
+			&bt_g_array_index(component->destroy_listeners,
 				struct bt_component_destroy_listener, i);
 
 		listener->func(component, listener->data);
@@ -161,11 +138,6 @@ void destroy_component(struct bt_object *obj)
 		finalize_component(component);
 	}
 
-	if (component->destroy) {
-		BT_LOGD_STR("Destroying type-specific data.");
-		component->destroy(component);
-	}
-
 	if (component->input_ports) {
 		BT_LOGD_STR("Destroying input ports.");
 		g_ptr_array_free(component->input_ports, TRUE);
@@ -193,34 +165,62 @@ void destroy_component(struct bt_object *obj)
 	g_free(component);
 }
 
+BT_EXPORT
 enum bt_component_class_type bt_component_get_class_type(
 		const struct bt_component *component)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(component, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL(component);
 	return component->class->type;
 }
 
+static
+bool port_name_is_unique(GPtrArray *ports, const char *name)
+{
+	guint i;
+	bool unique;
+
+	for (i = 0; i < ports->len; i++) {
+		struct bt_port *port = g_ptr_array_index(ports, i);
+
+		if (strcmp(port->name->str, name) == 0) {
+			unique = false;
+			goto end;
+		}
+	}
+
+	unique = true;
+
+end:
+	return unique;
+}
+
 static
 enum bt_self_component_add_port_status add_port(
 		struct bt_component *component, GPtrArray *ports,
 		enum bt_port_type port_type, const char *name, void *user_data,
-		struct bt_port **port)
+		struct bt_port **port, const char *api_func, bool input)
 {
 	struct bt_port *new_port = NULL;
 	struct bt_graph *graph = NULL;
 	enum bt_self_component_add_port_status status;
 
-	BT_ASSERT_PRE_NON_NULL(component, "Component");
-	BT_ASSERT_PRE_NON_NULL(name, "Name");
-	BT_ASSERT_PRE(strlen(name) > 0, "Name is empty");
+	BT_ASSERT_PRE_NO_ERROR_FROM_FUNC(api_func);
+	BT_ASSERT_PRE_COMP_NON_NULL_FROM_FUNC(api_func, component);
+	BT_ASSERT_PRE_NAME_NON_NULL_FROM_FUNC(api_func, name);
+	BT_ASSERT_PRE_FROM_FUNC(api_func,
+		input ? "input" : "output" "-port-name-is-unique",
+		port_name_is_unique(component->output_ports, name),
+		input ? "Input" : "Output"
+			" port name is not unique: name=\"%s\", %![comp-]c",
+		name, component);
+	BT_ASSERT_PRE_FROM_FUNC(api_func, "name-is-not-empty",
+		strlen(name) > 0, "Name is empty");
 	graph = bt_component_borrow_graph(component);
-	BT_ASSERT_PRE(
+	BT_ASSERT_PRE_FROM_FUNC(api_func, "graph-is-not-configured",
 		graph->config_state == BT_GRAPH_CONFIGURATION_STATE_CONFIGURING,
 		"Component's graph is already configured: "
 		"%![comp-]+c, %![graph-]+g", component, graph);
 
-	// TODO: Validate that the name is not already used.
-
 	BT_LIB_LOGI("Adding port to component: %![comp-]+c, "
 		"port-type=%s, port-name=\"%s\"", component,
 		bt_port_type_string(port_type), name);
@@ -273,21 +273,20 @@ end:
 	return status;
 }
 
-BT_HIDDEN
-uint64_t bt_component_get_input_port_count(const struct bt_component *comp)
+uint64_t bt_component_get_input_port_count(const struct bt_component *comp,
+		const char *api_func)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(comp, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL_FROM_FUNC(api_func, comp);
 	return (uint64_t) comp->input_ports->len;
 }
 
-BT_HIDDEN
-uint64_t bt_component_get_output_port_count(const struct bt_component *comp)
+uint64_t bt_component_get_output_port_count(const struct bt_component *comp,
+		const char *api_func)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(comp, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL_FROM_FUNC(api_func, comp);
 	return (uint64_t) comp->output_ports->len;
 }
 
-BT_HIDDEN
 int bt_component_create(struct bt_component_class *component_class,
 		const char *name, bt_logging_level log_level,
 		struct bt_component **user_component)
@@ -303,7 +302,7 @@ int bt_component_create(struct bt_component_class *component_class,
 	BT_LIB_LOGI("Creating empty component from component class: %![cc-]+C, "
 		"comp-name=\"%s\", log-level=%s", component_class, name,
 		bt_common_logging_level_string(log_level));
-	component = component_create_funcs[type](component_class);
+	component = component_create_funcs[type]();
 	if (!component) {
 		BT_LIB_LOGE_APPEND_CAUSE(
 			"Cannot create specific component object.");
@@ -314,7 +313,6 @@ int bt_component_create(struct bt_component_class *component_class,
 	bt_object_init_shared_with_parent(&component->base, destroy_component);
 	component->class = component_class;
 	bt_object_get_ref_no_null_check(component->class);
-	component->destroy = component_destroy_funcs[type];
 	component->name = g_string_new(name);
 	if (!component->name) {
 		BT_LIB_LOGE_APPEND_CAUSE("Failed to allocate one GString.");
@@ -356,38 +354,41 @@ end:
 	return ret;
 }
 
+BT_EXPORT
 const char *bt_component_get_name(const struct bt_component *component)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(component, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL(component);
 	return component->name->str;
 }
 
+BT_EXPORT
 const struct bt_component_class *bt_component_borrow_class_const(
 		const struct bt_component *component)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(component, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL(component);
 	return component->class;
 }
 
+BT_EXPORT
 void *bt_self_component_get_data(const struct bt_self_component *self_comp)
 {
 	struct bt_component *component = (void *) self_comp;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(component, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL(component);
 	return component->user_data;
 }
 
+BT_EXPORT
 void bt_self_component_set_data(struct bt_self_component *self_comp,
 		void *data)
 {
 	struct bt_component *component = (void *) self_comp;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(component, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL(component);
 	component->user_data = data;
 	BT_LIB_LOGD("Set component's user data: %!+c", component);
 }
 
-BT_HIDDEN
 void bt_component_set_graph(struct bt_component *component,
 		struct bt_graph *graph)
 {
@@ -397,12 +398,12 @@ void bt_component_set_graph(struct bt_component *component,
 
 static
 struct bt_port *borrow_port_by_name(GPtrArray *ports,
-		const char *name)
+		const char *name, const char *api_func)
 {
 	uint64_t i;
 	struct bt_port *ret_port = NULL;
 
-	BT_ASSERT(name);
+	BT_ASSERT_PRE_DEV_NAME_NON_NULL_FROM_FUNC(api_func, name);
 
 	for (i = 0; i < ports->len; i++) {
 		struct bt_port *port = g_ptr_array_index(ports, i);
@@ -416,71 +417,68 @@ struct bt_port *borrow_port_by_name(GPtrArray *ports,
 	return ret_port;
 }
 
-BT_HIDDEN
 struct bt_port_input *bt_component_borrow_input_port_by_name(
-		struct bt_component *comp, const char *name)
+		struct bt_component *comp, const char *name,
+		const char *api_func)
 {
-	BT_ASSERT(comp);
-	return (void *) borrow_port_by_name(comp->input_ports, name);
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL_FROM_FUNC(api_func, comp);
+	return (void *) borrow_port_by_name(comp->input_ports, name, api_func);
 }
 
-BT_HIDDEN
 struct bt_port_output *bt_component_borrow_output_port_by_name(
-		struct bt_component *comp, const char *name)
+		struct bt_component *comp, const char *name,
+		const char *api_func)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(comp, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL_FROM_FUNC(api_func, comp);
 	return (void *)
-		borrow_port_by_name(comp->output_ports, name);
+		borrow_port_by_name(comp->output_ports, name, api_func);
 }
 
 static
-struct bt_port *borrow_port_by_index(GPtrArray *ports, uint64_t index)
+struct bt_port *borrow_port_by_index(GPtrArray *ports, uint64_t index,
+		const char *api_func)
 {
-	BT_ASSERT(index < ports->len);
+	BT_ASSERT_PRE_DEV_VALID_INDEX_FROM_FUNC(api_func, index, ports->len);
 	return g_ptr_array_index(ports, index);
 }
 
-BT_HIDDEN
 struct bt_port_input *bt_component_borrow_input_port_by_index(
-		struct bt_component *comp, uint64_t index)
+		struct bt_component *comp, uint64_t index,
+		const char *api_func)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(comp, "Component");
-	BT_ASSERT_PRE_DEV_VALID_INDEX(index, comp->input_ports->len);
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL_FROM_FUNC(api_func, comp);
 	return (void *)
-		borrow_port_by_index(comp->input_ports, index);
+		borrow_port_by_index(comp->input_ports, index, api_func);
 }
 
-BT_HIDDEN
 struct bt_port_output *bt_component_borrow_output_port_by_index(
-		struct bt_component *comp, uint64_t index)
+		struct bt_component *comp, uint64_t index,
+		const char *api_func)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(comp, "Component");
-	BT_ASSERT_PRE_DEV_VALID_INDEX(index, comp->output_ports->len);
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL_FROM_FUNC(api_func, comp);
 	return (void *)
-		borrow_port_by_index(comp->output_ports, index);
+		borrow_port_by_index(comp->output_ports, index, api_func);
 }
 
-BT_HIDDEN
 enum bt_self_component_add_port_status bt_component_add_input_port(
 		struct bt_component *component, const char *name,
-		void *user_data, struct bt_port **port)
+		void *user_data, struct bt_port **port, const char *api_func)
 {
-	/* add_port() logs details */
+	/* add_port() logs details and checks preconditions */
 	return add_port(component, component->input_ports,
-		BT_PORT_TYPE_INPUT, name, user_data, port);
+		BT_PORT_TYPE_INPUT, name, user_data, port, api_func, true);
 }
 
-BT_HIDDEN
 enum bt_self_component_add_port_status bt_component_add_output_port(
 		struct bt_component *component, const char *name,
-		void *user_data, struct bt_port **port)
+		void *user_data, struct bt_port **port,
+		const char *api_func)
 {
-	/* add_port() logs details */
+	/* add_port() logs details and checks preconditions */
 	return add_port(component, component->output_ports,
-		BT_PORT_TYPE_OUTPUT, name, user_data, port);
+		BT_PORT_TYPE_OUTPUT, name, user_data, port, api_func, false);
 }
 
-BT_HIDDEN
 enum bt_component_class_port_connected_method_status
 bt_component_port_connected(
 		struct bt_component *comp, struct bt_port *self_port,
@@ -492,6 +490,7 @@ bt_component_port_connected(
 	enum bt_component_class_port_connected_method_status status =
 		BT_FUNC_STATUS_OK;
 	method_t method = NULL;
+	const char *method_name = NULL;
 
 	BT_ASSERT(comp);
 	BT_ASSERT(self_port);
@@ -505,6 +504,7 @@ bt_component_port_connected(
 		switch (self_port->type) {
 		case BT_PORT_TYPE_OUTPUT:
 			method = (method_t) src_cc->methods.output_port_connected;
+			method_name = "bt_component_class_source_output_port_connected_method";
 			break;
 		default:
 			bt_common_abort();
@@ -519,9 +519,11 @@ bt_component_port_connected(
 		switch (self_port->type) {
 		case BT_PORT_TYPE_INPUT:
 			method = (method_t) flt_cc->methods.input_port_connected;
+			method_name = "bt_component_class_filter_input_port_connected_method";
 			break;
 		case BT_PORT_TYPE_OUTPUT:
 			method = (method_t) flt_cc->methods.output_port_connected;
+			method_name = "bt_component_class_filter_output_port_connected_method";
 			break;
 		default:
 			bt_common_abort();
@@ -536,6 +538,7 @@ bt_component_port_connected(
 		switch (self_port->type) {
 		case BT_PORT_TYPE_INPUT:
 			method = (method_t) sink_cc->methods.input_port_connected;
+			method_name = "bt_component_class_sink_input_port_connected_method";
 			break;
 		default:
 			bt_common_abort();
@@ -554,18 +557,18 @@ bt_component_port_connected(
 		status = (int) method(comp, self_port, (void *) other_port);
 		BT_LOGD("User method returned: status=%s",
 			bt_common_func_status_string(status));
-		BT_ASSERT_POST(status == BT_FUNC_STATUS_OK ||
+		BT_ASSERT_POST(method_name, "valid-status",
+			status == BT_FUNC_STATUS_OK ||
 			status == BT_FUNC_STATUS_ERROR ||
 			status == BT_FUNC_STATUS_MEMORY_ERROR,
 			"Unexpected returned component status: status=%s",
 			bt_common_func_status_string(status));
-		BT_ASSERT_POST_NO_ERROR_IF_NO_ERROR_STATUS(status);
+		BT_ASSERT_POST_NO_ERROR_IF_NO_ERROR_STATUS(method_name, status);
 	}
 
 	return status;
 }
 
-BT_HIDDEN
 void bt_component_add_destroy_listener(struct bt_component *component,
 		bt_component_destroy_listener_func func, void *data)
 {
@@ -581,7 +584,6 @@ void bt_component_add_destroy_listener(struct bt_component *component,
 		component, func, data);
 }
 
-BT_HIDDEN
 void bt_component_remove_destroy_listener(struct bt_component *component,
 		bt_component_destroy_listener_func func, void *data)
 {
@@ -592,7 +594,7 @@ void bt_component_remove_destroy_listener(struct bt_component *component,
 
 	for (i = 0; i < component->destroy_listeners->len; i++) {
 		struct bt_component_destroy_listener *listener =
-			&g_array_index(component->destroy_listeners,
+			&bt_g_array_index(component->destroy_listeners,
 				struct bt_component_destroy_listener, i);
 
 		if (listener->func == func && listener->data == data) {
@@ -605,27 +607,31 @@ void bt_component_remove_destroy_listener(struct bt_component *component,
 	}
 }
 
+BT_EXPORT
 bt_logging_level bt_component_get_logging_level(
 		const struct bt_component *component)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(component, "Component");
+	BT_ASSERT_PRE_DEV_COMP_NON_NULL(component);
 	return component->log_level;
 }
 
+BT_EXPORT
 uint64_t bt_self_component_get_graph_mip_version(
 		bt_self_component *self_component)
 {
 	struct bt_component *comp = (void *) self_component;
 
-	BT_ASSERT_PRE_NON_NULL(self_component, "Component");
+	BT_ASSERT_PRE_COMP_NON_NULL(self_component);
 	return bt_component_borrow_graph(comp)->mip_version;
 }
 
+BT_EXPORT
 void bt_component_get_ref(const struct bt_component *component)
 {
 	bt_object_get_ref(component);
 }
 
+BT_EXPORT
 void bt_component_put_ref(const struct bt_component *component)
 {
 	bt_object_put_ref(component);
