@@ -1,33 +1,14 @@
 /*
+ * SPDX-License-Identifier: MIT
+ *
  * Copyright 2017-2018 Philippe Proulx <pproulx@efficios.com>
  * Copyright 2016 Jérémie Galarneau <jeremie.galarneau@efficios.com>
- *
- * Author: Jérémie Galarneau <jeremie.galarneau@efficios.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
  */
 
 #define BT_LOG_TAG "LIB/PLUGIN-SO"
 #include "lib/logging.h"
 
 #include "common/assert.h"
-#include "lib/assert-pre.h"
 #include "compat/compiler.h"
 #include <babeltrace2/plugin/plugin-dev.h>
 #include "lib/graph/component-class.h"
@@ -84,6 +65,11 @@ BT_PLUGIN_MODULE();
  * component class is removed from this list, the shared library handle
  * object's reference count falls to zero and the shared library is
  * finally closed.
+ *
+ * We're not using a GLib linked list here because this destructor is
+ * called after GLib's thread-specific data is destroyed, which contains
+ * the allocated memory for GLib data structures (what's used by
+ * g_slice_alloc()).
  */
 
 static
@@ -171,7 +157,7 @@ int bt_plugin_so_shared_lib_handle_create(
 	int status = BT_FUNC_STATUS_OK;
 
 	BT_ASSERT(shared_lib_handle);
-	BT_LOGI("Creating shared library handle: path=\"%s\"", path);
+	BT_LOGI("Creating shared library handle: path=\"%s\"", path ? path : "(null)");
 	*shared_lib_handle = g_new0(struct bt_plugin_so_shared_lib_handle, 1);
 	if (!*shared_lib_handle) {
 		BT_LIB_LOGE_APPEND_CAUSE("Failed to allocate one shared library handle.");
@@ -215,7 +201,7 @@ end:
 	BT_ASSERT(*shared_lib_handle || status != BT_FUNC_STATUS_OK);
 	if (*shared_lib_handle) {
 		BT_LOGI("Created shared library handle: path=\"%s\", addr=%p",
-			path, *shared_lib_handle);
+			path ? path : "(null)", *shared_lib_handle);
 	}
 
 	return status;
@@ -473,7 +459,7 @@ int bt_plugin_so_init(struct bt_plugin *plugin,
 		/* Find the corresponding component class descriptor entry */
 		for (i = 0; i < comp_class_full_descriptors->len; i++) {
 			struct comp_class_full_descriptor *cc_full_descr =
-				&g_array_index(comp_class_full_descriptors,
+				&bt_g_array_index(comp_class_full_descriptors,
 					struct comp_class_full_descriptor, i);
 
 			if (cur_cc_descr_attr->comp_class_descriptor !=
@@ -699,7 +685,7 @@ int bt_plugin_so_init(struct bt_plugin *plugin,
 							NULL,
 						descriptor->name,
 						cur_cc_descr_attr->comp_class_descriptor->name,
-						bt_component_class_type_string(
+						bt_common_component_class_type_string(
 							cur_cc_descr_attr->comp_class_descriptor->type),
 						cur_cc_descr_attr->type_name,
 						cur_cc_descr_attr->type);
@@ -719,7 +705,7 @@ int bt_plugin_so_init(struct bt_plugin *plugin,
 							NULL,
 						descriptor->name,
 						cur_cc_descr_attr->comp_class_descriptor->name,
-						bt_component_class_type_string(
+						bt_common_component_class_type_string(
 							cur_cc_descr_attr->comp_class_descriptor->type),
 						cur_cc_descr_attr->type_name,
 						cur_cc_descr_attr->type);
@@ -748,6 +734,12 @@ int bt_plugin_so_init(struct bt_plugin *plugin,
 				status = init_status;
 				goto end;
 			} else {
+				/*
+				 * Since we don't return an error,
+				 * there's no way to communicate this
+				 * error to the caller.
+				 */
+				bt_current_thread_clear_error();
 				BT_LIB_LOGW(
 					"User's plugin initialization function failed: "
 					"status=%s",
@@ -764,7 +756,7 @@ int bt_plugin_so_init(struct bt_plugin *plugin,
 	/* Add described component classes to plugin */
 	for (i = 0; i < comp_class_full_descriptors->len; i++) {
 		struct comp_class_full_descriptor *cc_full_descr =
-			&g_array_index(comp_class_full_descriptors,
+			&bt_g_array_index(comp_class_full_descriptors,
 				struct comp_class_full_descriptor, i);
 		struct bt_component_class *comp_class = NULL;
 		struct bt_component_class_source *src_comp_class = NULL;
@@ -779,7 +771,7 @@ int bt_plugin_so_init(struct bt_plugin *plugin,
 				NULL,
 			descriptor->name,
 			cc_full_descr->descriptor->name,
-			bt_component_class_type_string(
+			bt_common_component_class_type_string(
 				cc_full_descr->descriptor->type));
 
 		if (cc_full_descr->descriptor->type == BT_COMPONENT_CLASS_TYPE_SOURCE ||
@@ -1349,6 +1341,12 @@ int bt_plugin_so_create_all_from_sections(
 			/* Add to plugin set */
 			bt_plugin_set_add_plugin(*plugin_set_out, plugin);
 			BT_OBJECT_PUT_REF_AND_RESET(plugin);
+		} else if (status == BT_FUNC_STATUS_NOT_FOUND) {
+			/*
+			 * There was an error initializing the plugin,
+			 * but `fail_on_load_error` is false.
+			 */
+		        BT_OBJECT_PUT_REF_AND_RESET(plugin);
 		} else if (status < 0) {
 			/*
 			 * bt_plugin_so_init() handles
@@ -1381,7 +1379,6 @@ end:
 	return status;
 }
 
-BT_HIDDEN
 int bt_plugin_so_create_all_from_static(bool fail_on_load_error,
 		struct bt_plugin_set **plugin_set_out)
 {
@@ -1418,7 +1415,6 @@ end:
 	return status;
 }
 
-BT_HIDDEN
 int bt_plugin_so_create_all_from_file(const char *path,
 		bool fail_on_load_error, struct bt_plugin_set **plugin_set_out)
 {
@@ -1677,7 +1673,7 @@ end:
 
 static
 void plugin_comp_class_destroy_listener(struct bt_component_class *comp_class,
-		void *data)
+		void *data __attribute__((unused)))
 {
 	bt_list_del(&comp_class->node);
 	BT_OBJECT_PUT_REF_AND_RESET(comp_class->so_handle);
@@ -1685,6 +1681,13 @@ void plugin_comp_class_destroy_listener(struct bt_component_class *comp_class,
 		"comp-cls-addr=%p", comp_class);
 }
 
+/*
+ * This function would normally not be BT_EXPORTed, but it is used by the
+ * Python plugin provider, which is conceptually part of libbabeltrace2, but
+ * implemented as a separate shared object, for modularity.  It is therefore
+ * exposed, but not part of the public ABI.
+ */
+BT_EXPORT
 void bt_plugin_so_on_add_component_class(struct bt_plugin *plugin,
 		struct bt_component_class *comp_class)
 {
