@@ -1,35 +1,18 @@
 /*
+ * SPDX-License-Identifier: MIT
+ *
  * Copyright 2017-2018 Philippe Proulx <pproulx@efficios.com>
  * Copyright 2013, 2014 Jérémie Galarneau <jeremie.galarneau@efficios.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
  */
 
 #define BT_LOG_TAG "LIB/FIELD"
 #include "lib/logging.h"
 
-#include "lib/assert-pre.h"
+#include "lib/assert-cond.h"
 #include <babeltrace2/trace-ir/field.h>
 #include "lib/object.h"
 #include "compat/compiler.h"
 #include "compat/fcntl.h"
-#include "common/align.h"
 #include "common/assert.h"
 #include <inttypes.h>
 #include <stdbool.h>
@@ -37,6 +20,11 @@
 #include "field.h"
 #include "field-class.h"
 #include "lib/func-status.h"
+#include "utils.h"
+
+#define BT_ASSERT_PRE_DEV_FIELD_HOT(_field)				\
+	BT_ASSERT_PRE_DEV_HOT("field",					\
+		(const struct bt_field *) (_field), "Field", ": %!+f", (_field))
 
 static
 void reset_single_field(struct bt_field *field);
@@ -146,6 +134,13 @@ struct bt_field_methods variant_field_methods = {
 	.reset = reset_variant_field,
 };
 
+static
+struct bt_field_methods blob_field_methods = {
+	.set_is_frozen = set_single_field_is_frozen,
+	.is_set = single_field_is_set,
+	.reset = reset_single_field,
+};
+
 static
 struct bt_field *create_bool_field(struct bt_field_class *);
 
@@ -176,6 +171,9 @@ struct bt_field *create_option_field(struct bt_field_class *);
 static
 struct bt_field *create_variant_field(struct bt_field_class *);
 
+static
+struct bt_field *create_blob_field(struct bt_field_class *);
+
 static
 void destroy_bool_field(struct bt_field *field);
 
@@ -203,26 +201,31 @@ void destroy_option_field(struct bt_field *field);
 static
 void destroy_variant_field(struct bt_field *field);
 
+static
+void destroy_blob_field(struct bt_field *field);
+
+BT_EXPORT
 struct bt_field_class *bt_field_borrow_class(struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
 	return field->class;
 }
 
+BT_EXPORT
 const struct bt_field_class *bt_field_borrow_class_const(
 		const struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
 	return field->class;
 }
 
+BT_EXPORT
 enum bt_field_class_type bt_field_get_class_type(const struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
 	return field->class->type;
 }
 
-BT_HIDDEN
 struct bt_field *bt_field_create(struct bt_field_class *fc)
 {
 	struct bt_field *field = NULL;
@@ -270,6 +273,11 @@ struct bt_field *bt_field_create(struct bt_field_class *fc)
 	case BT_FIELD_CLASS_TYPE_VARIANT_WITH_SIGNED_INTEGER_SELECTOR_FIELD:
 		field = create_variant_field(fc);
 		break;
+	case BT_FIELD_CLASS_TYPE_STATIC_BLOB:
+	case BT_FIELD_CLASS_TYPE_DYNAMIC_BLOB_WITHOUT_LENGTH_FIELD:
+	case BT_FIELD_CLASS_TYPE_DYNAMIC_BLOB_WITH_LENGTH_FIELD:
+		field = create_blob_field(fc);
+		break;
 	default:
 		bt_common_abort();
 	}
@@ -398,7 +406,8 @@ struct bt_field *create_string_field(struct bt_field_class *fc)
 		goto end;
 	}
 
-	g_array_index(string_field->buf, char, 0) = '\0';
+	g_array_set_size(string_field->buf, 1);
+	bt_g_array_index(string_field->buf, char, 0) = '\0';
 	BT_LIB_LOGD("Created string field object: %!+f", string_field);
 
 end:
@@ -536,6 +545,43 @@ end:
 	return (void *) var_field;
 }
 
+static
+struct bt_field *create_blob_field(struct bt_field_class *fc)
+{
+	struct bt_field_blob *blob_field;
+
+	BT_LIB_LOGD("Creating BLOB field object: %![fc-]+F", fc);
+	blob_field = g_new0(struct bt_field_blob, 1);
+	if (!blob_field) {
+		BT_LIB_LOGE_APPEND_CAUSE(
+			"Failed to allocate one BLOB field.");
+		goto end;
+	}
+
+	init_field((void *) blob_field, fc, &blob_field_methods);
+
+	if (bt_field_class_type_is(fc->type, BT_FIELD_CLASS_TYPE_STATIC_BLOB)) {
+		struct bt_field_class_blob_static *blob_static_fc =
+			(void *) fc;
+		blob_field->length = blob_static_fc->length;
+		blob_field->data = g_malloc(blob_field->length);
+		if (!blob_field->data) {
+			BT_LIB_LOGE_APPEND_CAUSE(
+				"Failed to allocate BLOB field data: %![fc-]+F",
+				fc);
+			goto error;
+		}
+	}
+
+	goto end;
+error:
+	bt_field_destroy((void *) blob_field);
+	blob_field = NULL;
+
+end:
+	return (void *) blob_field;
+}
+
 static inline
 int init_array_field_fields(struct bt_field_array *array_field)
 {
@@ -632,50 +678,54 @@ end:
 	return (void *) array_field;
 }
 
+BT_EXPORT
 bt_bool bt_field_bool_get_value(const struct bt_field *field)
 {
 	const struct bt_field_bool *bool_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field, BT_FIELD_CLASS_TYPE_BOOL,
-		"Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"boolean-field", BT_FIELD_CLASS_TYPE_BOOL, "Field");
 	return (bt_bool) bool_field->value;
 }
 
+BT_EXPORT
 void bt_field_bool_set_value(struct bt_field *field, bt_bool value)
 {
 	struct bt_field_bool *bool_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field, BT_FIELD_CLASS_TYPE_BOOL,
-		"Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"boolean-field", BT_FIELD_CLASS_TYPE_BOOL, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 	bool_field->value = (bool) value;
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
 uint64_t bt_field_bit_array_get_value_as_integer(const struct bt_field *field)
 {
 	const struct bt_field_bit_array *ba_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_BIT_ARRAY, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"bit-array-field", BT_FIELD_CLASS_TYPE_BIT_ARRAY, "Field");
 	return ba_field->value_as_int;
 }
 
+BT_EXPORT
 void bt_field_bit_array_set_value_as_integer(struct bt_field *field,
 		uint64_t value)
 {
 	struct bt_field_bit_array *ba_field = (void *) field;
 	struct bt_field_class_bit_array *ba_fc;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_BIT_ARRAY, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"bit-array-field", BT_FIELD_CLASS_TYPE_BIT_ARRAY, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 	ba_fc = (void *) field->class;
 	ba_field->value_as_int = value;
 
@@ -687,107 +737,144 @@ void bt_field_bit_array_set_value_as_integer(struct bt_field *field,
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
+bt_field_bit_array_get_active_flag_labels_status
+bt_field_bit_array_get_active_flag_labels(const struct bt_field *field,
+		bt_field_class_bit_array_flag_label_array *label_array,
+		uint64_t *count)
+{
+	const struct bt_field_bit_array *ba_field = (const void *) field;
+
+	BT_ASSERT_PRE_DEV_NO_ERROR();
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"bit-array-field", BT_FIELD_CLASS_TYPE_BIT_ARRAY, "Field");
+	BT_ASSERT_PRE_DEV_NON_NULL("label-array-output", label_array,
+		"Label array (output)");
+	BT_ASSERT_PRE_DEV_NON_NULL("count-output", count, "Count (output)");
+	return (int)
+		bt_field_class_bit_array_get_active_flag_labels_for_value_as_integer(
+			field->class, ba_field->value_as_int, label_array, count);
+}
+
+BT_EXPORT
 int64_t bt_field_integer_signed_get_value(const struct bt_field *field)
 {
 	const struct bt_field_integer *int_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SIGNED_INT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SIGNED_INT("field", field, "Field");
 	return int_field->value.i;
 }
 
+BT_EXPORT
 void bt_field_integer_signed_set_value(struct bt_field *field, int64_t value)
 {
 	struct bt_field_integer *int_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SIGNED_INT(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
-	BT_ASSERT_PRE_DEV(bt_util_value_is_in_range_signed(
-		((struct bt_field_class_integer *) field->class)->range, value),
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SIGNED_INT("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
+	BT_ASSERT_PRE_DEV("valid-value-for-field-class-field-value-range",
+		bt_util_value_is_in_range_signed(
+			((struct bt_field_class_integer *) field->class)->range,
+			value),
 		"Value is out of bounds: value=%" PRId64 ", %![field-]+f, "
 		"%![fc-]+F", value, field, field->class);
 	int_field->value.i = value;
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
 uint64_t bt_field_integer_unsigned_get_value(const struct bt_field *field)
 {
 	const struct bt_field_integer *int_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_UNSIGNED_INT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_UNSIGNED_INT("field", field, "Field");
 	return int_field->value.u;
 }
 
+BT_EXPORT
 void bt_field_integer_unsigned_set_value(struct bt_field *field, uint64_t value)
 {
 	struct bt_field_integer *int_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_UNSIGNED_INT(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
-	BT_ASSERT_PRE_DEV(bt_util_value_is_in_range_unsigned(
-		((struct bt_field_class_integer *) field->class)->range, value),
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_UNSIGNED_INT("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
+	BT_ASSERT_PRE_DEV("valid-value-for-field-class-field-value-range",
+		bt_util_value_is_in_range_unsigned(
+			((struct bt_field_class_integer *) field->class)->range,
+			value),
 		"Value is out of bounds: value=%" PRIu64 ", %![field-]+f, "
 		"%![fc-]+F", value, field, field->class);
 	int_field->value.u = value;
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
 float bt_field_real_single_precision_get_value(const struct bt_field *field)
 {
 	const struct bt_field_real *real_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"single-precision-real-field",
 		BT_FIELD_CLASS_TYPE_SINGLE_PRECISION_REAL, "Field");
 	return (float) real_field->value;
 }
 
+BT_EXPORT
 double bt_field_real_double_precision_get_value(const struct bt_field *field)
 {
 	const struct bt_field_real *real_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"double-precision-real-field",
 		BT_FIELD_CLASS_TYPE_DOUBLE_PRECISION_REAL, "Field");
-
 	return real_field->value;
 }
 
+BT_EXPORT
 void bt_field_real_single_precision_set_value(struct bt_field *field,
 		float value)
 {
 	struct bt_field_real *real_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"single-precision-real-field",
 		BT_FIELD_CLASS_TYPE_SINGLE_PRECISION_REAL, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 
 	real_field->value = (double) value;
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
 void bt_field_real_double_precision_set_value(struct bt_field *field,
 		double value)
 {
 	struct bt_field_real *real_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"double-precision-real-field",
 		BT_FIELD_CLASS_TYPE_DOUBLE_PRECISION_REAL, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 
 	real_field->value = value;
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
 enum bt_field_enumeration_get_mapping_labels_status
 bt_field_enumeration_unsigned_get_mapping_labels(
 		const struct bt_field *field,
@@ -797,17 +884,20 @@ bt_field_enumeration_unsigned_get_mapping_labels(
 	const struct bt_field_integer *int_field = (const void *) field;
 
 	BT_ASSERT_PRE_DEV_NO_ERROR();
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_NON_NULL(label_array, "Label array (output)");
-	BT_ASSERT_PRE_DEV_NON_NULL(label_array, "Count (output)");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_NON_NULL("label-array-output", label_array,
+		"Label array (output)");
+	BT_ASSERT_PRE_DEV_NON_NULL("count-output", count, "Count (output)");
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"unsigned-enumeration-field",
 		BT_FIELD_CLASS_TYPE_UNSIGNED_ENUMERATION, "Field");
 	return (int)
 		bt_field_class_enumeration_unsigned_get_mapping_labels_for_value(
 			field->class, int_field->value.u, label_array, count);
 }
 
+BT_EXPORT
 enum bt_field_enumeration_get_mapping_labels_status
 bt_field_enumeration_signed_get_mapping_labels(
 		const struct bt_field *field,
@@ -817,36 +907,40 @@ bt_field_enumeration_signed_get_mapping_labels(
 	const struct bt_field_integer *int_field = (const void *) field;
 
 	BT_ASSERT_PRE_DEV_NO_ERROR();
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_NON_NULL(label_array, "Label array (output)");
-	BT_ASSERT_PRE_DEV_NON_NULL(label_array, "Count (output)");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_NON_NULL("label-array-output", label_array,
+		"Label array (output)");
+	BT_ASSERT_PRE_DEV_NON_NULL("count-output", count, "Count (output)");
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"signed-enumeration-field",
 		BT_FIELD_CLASS_TYPE_SIGNED_ENUMERATION, "Field");
 	return (int)
 		bt_field_class_enumeration_signed_get_mapping_labels_for_value(
 			field->class, int_field->value.i, label_array, count);
 }
 
+BT_EXPORT
 const char *bt_field_string_get_value(const struct bt_field *field)
 {
 	const struct bt_field_string *string_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field, BT_FIELD_CLASS_TYPE_STRING,
-		"Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field, "string-field",
+		BT_FIELD_CLASS_TYPE_STRING, "Field");
 	return (const char *) string_field->buf->data;
 }
 
+BT_EXPORT
 uint64_t bt_field_string_get_length(const struct bt_field *field)
 {
 	const struct bt_field_string *string_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_SET(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field, BT_FIELD_CLASS_TYPE_STRING,
-		"Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_SET("field", field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field, "string-field",
+		BT_FIELD_CLASS_TYPE_STRING, "Field");
 	return string_field->length;
 }
 
@@ -857,52 +951,51 @@ void clear_string_field(struct bt_field *field)
 
 	BT_ASSERT_DBG(field);
 	string_field->length = 0;
-	g_array_index(string_field->buf, char, 0) = '\0';
+	bt_g_array_index(string_field->buf, char, 0) = '\0';
 	bt_field_set_single(field, true);
 }
 
+BT_EXPORT
 enum bt_field_string_set_value_status bt_field_string_set_value(
 		struct bt_field *field, const char *value)
 {
 	BT_ASSERT_PRE_DEV_NO_ERROR();
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_NON_NULL(value, "Value");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field, BT_FIELD_CLASS_TYPE_STRING,
-		"Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_NON_NULL("value", value, "Value");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field, "string-field",
+		BT_FIELD_CLASS_TYPE_STRING, "Field");
 	clear_string_field(field);
 	return (int) bt_field_string_append_with_length(field, value,
 		(uint64_t) strlen(value));
 }
 
-enum bt_field_string_append_status bt_field_string_append(
-		struct bt_field *field, const char *value)
-{
-	BT_ASSERT_PRE_DEV_NO_ERROR();
+#define BT_ASSERT_PRE_DEV_FOR_APPEND_TO_STRING_FIELD_WITH_LENGTH(_field, _value, _length) \
+	do {								\
+		BT_ASSERT_PRE_DEV_NO_ERROR();				\
+		BT_ASSERT_PRE_DEV_FIELD_NON_NULL(_field);		\
+		BT_ASSERT_PRE_DEV_NON_NULL("value", (_value), "Value");	\
+		BT_ASSERT_PRE_DEV_FIELD_HOT(_field);			\
+		BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field",		\
+			(_field), "string-field",			\
+			BT_FIELD_CLASS_TYPE_STRING, "Field");		\
+		BT_ASSERT_PRE_DEV("value-has-no-null-byte",		\
+			!memchr((_value), '\0', (_length)),		\
+			"String value to append contains a null character: " \
+			"partial-value=\"%.32s\", length=%" PRIu64,	\
+			(_value), (_length));					\
+	} while (0)
 
-	return bt_field_string_append_with_length(field,
-		value, (uint64_t) strlen(value));
-}
-
-enum bt_field_string_append_status bt_field_string_append_with_length(
+static
+enum bt_field_string_append_status append_to_string_field_with_length(
 		struct bt_field *field, const char *value, uint64_t length)
 {
 	struct bt_field_string *string_field = (void *) field;
 	char *data;
 	uint64_t new_length;
 
-	BT_ASSERT_PRE_DEV_NO_ERROR();
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_NON_NULL(value, "Value");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_STRING, "Field");
-
-	/* Make sure no null bytes are appended */
-	BT_ASSERT_PRE_DEV(!memchr(value, '\0', length),
-		"String value to append contains a null character: "
-		"partial-value=\"%.32s\", length=%" PRIu64, value, length);
-
+	BT_ASSERT_DBG(field);
+	BT_ASSERT_DBG(value);
 	new_length = length + string_field->length;
 
 	if (G_UNLIKELY(new_length + 1 > string_field->buf->len)) {
@@ -917,24 +1010,47 @@ enum bt_field_string_append_status bt_field_string_append_with_length(
 	return BT_FUNC_STATUS_OK;
 }
 
+BT_EXPORT
+enum bt_field_string_append_status bt_field_string_append_with_length(
+		struct bt_field *field, const char *value, uint64_t length)
+{
+	BT_ASSERT_PRE_DEV_FOR_APPEND_TO_STRING_FIELD_WITH_LENGTH(field, value,
+		length);
+	return append_to_string_field_with_length(field, value, length);
+}
+
+BT_EXPORT
+enum bt_field_string_append_status bt_field_string_append(
+		struct bt_field *field, const char *value)
+{
+	uint64_t length = (uint64_t) strlen(value);
+
+	BT_ASSERT_PRE_DEV_FOR_APPEND_TO_STRING_FIELD_WITH_LENGTH(field, value,
+		length);
+	return append_to_string_field_with_length(field, value, length);
+}
+
+BT_EXPORT
 void bt_field_string_clear(struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field, "string-field",
 		BT_FIELD_CLASS_TYPE_STRING, "Field");
 	clear_string_field(field);
 }
 
+BT_EXPORT
 uint64_t bt_field_array_get_length(const struct bt_field *field)
 {
 	const struct bt_field_array *array_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_ARRAY(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_ARRAY("field", field, "Field");
 	return array_field->length;
 }
 
+BT_EXPORT
 enum bt_field_array_dynamic_set_length_status bt_field_array_dynamic_set_length(
 		struct bt_field *field, uint64_t length)
 {
@@ -942,9 +1058,9 @@ enum bt_field_array_dynamic_set_length_status bt_field_array_dynamic_set_length(
 	struct bt_field_array *array_field = (void *) field;
 
 	BT_ASSERT_PRE_DEV_NO_ERROR();
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_DYNAMIC_ARRAY(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_DYNAMIC_ARRAY("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 
 	if (G_UNLIKELY(length > array_field->fields->len)) {
 		/* Make more room */
@@ -982,60 +1098,70 @@ end:
 
 static inline
 struct bt_field *borrow_array_field_element_field_by_index(
-		struct bt_field *field, uint64_t index)
+		struct bt_field *field, uint64_t index, const char *api_func)
 {
 	struct bt_field_array *array_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_ARRAY(field, "Field");
-	BT_ASSERT_PRE_DEV_VALID_INDEX(index, array_field->length);
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL_FROM_FUNC(api_func, field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_ARRAY_FROM_FUNC(api_func, "field", field,
+		"Field");
+	BT_ASSERT_PRE_DEV_VALID_INDEX_FROM_FUNC(api_func, index,
+		array_field->length);
 	return array_field->fields->pdata[index];
 }
 
+BT_EXPORT
 struct bt_field *bt_field_array_borrow_element_field_by_index(
 		struct bt_field *field, uint64_t index)
 {
-	return borrow_array_field_element_field_by_index(field, index);
+	return borrow_array_field_element_field_by_index(field, index,
+		__func__);
 }
 
+BT_EXPORT
 const struct bt_field *
 bt_field_array_borrow_element_field_by_index_const(
 		const struct bt_field *field, uint64_t index)
 {
-	return borrow_array_field_element_field_by_index((void *) field, index);
+	return borrow_array_field_element_field_by_index((void *) field, index,
+		__func__);
 }
 
 static inline
 struct bt_field *borrow_structure_field_member_field_by_index(
-		struct bt_field *field, uint64_t index)
+		struct bt_field *field, uint64_t index, const char *api_func)
 {
 	struct bt_field_structure *struct_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_STRUCTURE, "Field");
-	BT_ASSERT_PRE_DEV_VALID_INDEX(index, struct_field->fields->len);
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL_FROM_FUNC(api_func, field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE_FROM_FUNC(api_func, "field",
+		field, "structure-field", BT_FIELD_CLASS_TYPE_STRUCTURE,
+		"Field");
+	BT_ASSERT_PRE_DEV_VALID_INDEX_FROM_FUNC(api_func, index,
+		struct_field->fields->len);
 	return struct_field->fields->pdata[index];
 }
 
+BT_EXPORT
 struct bt_field *bt_field_structure_borrow_member_field_by_index(
 		struct bt_field *field, uint64_t index)
 {
 	return borrow_structure_field_member_field_by_index(field,
-		index);
+		index, __func__);
 }
 
+BT_EXPORT
 const struct bt_field *
 bt_field_structure_borrow_member_field_by_index_const(
 		const struct bt_field *field, uint64_t index)
 {
 	return borrow_structure_field_member_field_by_index(
-		(void *) field, index);
+		(void *) field, index, __func__);
 }
 
 static inline
 struct bt_field *borrow_structure_field_member_field_by_name(
-		struct bt_field *field, const char *name)
+		struct bt_field *field, const char *name, const char *api_func)
 {
 	struct bt_field *ret_field = NULL;
 	struct bt_field_class_structure *struct_fc;
@@ -1043,10 +1169,12 @@ struct bt_field *borrow_structure_field_member_field_by_name(
 	gpointer orig_key;
 	gpointer index;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_NON_NULL(name, "Field name");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_STRUCTURE, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL_FROM_FUNC(api_func, field);
+	BT_ASSERT_PRE_DEV_NON_NULL_FROM_FUNC(api_func, "member-name", name,
+		"Member name");
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE_FROM_FUNC(api_func, "field",
+		field, "structure-field", BT_FIELD_CLASS_TYPE_STRUCTURE,
+		"Field");
 	struct_fc = (void *) field->class;
 
 	if (!g_hash_table_lookup_extended(struct_fc->common.name_to_index, name,
@@ -1061,26 +1189,30 @@ end:
 	return ret_field;
 }
 
+BT_EXPORT
 struct bt_field *bt_field_structure_borrow_member_field_by_name(
 		struct bt_field *field, const char *name)
 {
-	return borrow_structure_field_member_field_by_name(field, name);
+	return borrow_structure_field_member_field_by_name(field, name,
+		__func__);
 }
 
+BT_EXPORT
 const struct bt_field *bt_field_structure_borrow_member_field_by_name_const(
 		const struct bt_field *field, const char *name)
 {
 	return borrow_structure_field_member_field_by_name(
-		(void *) field, name);
+		(void *) field, name, __func__);
 }
 
+BT_EXPORT
 void bt_field_option_set_has_field(struct bt_field *field, bt_bool has_field)
 {
 	struct bt_field_option *opt_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_OPTION(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_OPTION("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 
 	if (has_field) {
 		opt_field->selected_field = opt_field->content_field;
@@ -1089,46 +1221,70 @@ void bt_field_option_set_has_field(struct bt_field *field, bt_bool has_field)
 	}
 }
 
+BT_EXPORT
 struct bt_field *bt_field_option_borrow_field(struct bt_field *field)
 {
 	struct bt_field_option *opt_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_OPTION(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_OPTION("field", field, "Field");
 	return opt_field->selected_field;
 }
 
+BT_EXPORT
 const struct bt_field *bt_field_option_borrow_field_const(
 		const struct bt_field *field)
 {
 	return (const void *) bt_field_option_borrow_field((void *) field);
 }
 
+#define BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_OPT_FIELD(_field)	\
+	do {								\
+		struct bt_field_variant *_var_field = (void *) field;	\
+		BT_ASSERT_PRE_DEV_FIELD_NON_NULL(_field);		\
+		BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT("field", (_field),	\
+			"Field");					\
+		BT_ASSERT_PRE_DEV("has-selected-field",			\
+			_var_field->selected_field,			\
+			"Variant field has no selected field: %!+f",	\
+			field);						\
+	} while (0)
+
 static inline
 struct bt_field *borrow_variant_field_selected_option_field(
 		struct bt_field *field)
 {
 	struct bt_field_variant *var_field = (void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT(field, "Field");
-	BT_ASSERT_PRE_DEV(var_field->selected_field,
-		"Variant field has no selected field: %!+f", field);
+	BT_ASSERT_DBG(field);
 	return var_field->selected_field;
 }
 
+BT_EXPORT
 struct bt_field *bt_field_variant_borrow_selected_option_field(
 		struct bt_field *field)
 {
+	BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_OPT_FIELD(field);
 	return borrow_variant_field_selected_option_field(field);
 }
 
+BT_EXPORT
 const struct bt_field *bt_field_variant_borrow_selected_option_field_const(
 		const struct bt_field *field)
 {
+	BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_OPT_FIELD(field);
 	return borrow_variant_field_selected_option_field((void *) field);
 }
 
+#define BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_CLASS_OPT(_field)	\
+	do {								\
+		struct bt_field_variant *_var_field = (void *) field;	\
+		BT_ASSERT_PRE_DEV("has-selected-field",			\
+			_var_field->selected_field,			\
+			"Variant field has no selected field: %!+f",	\
+			(_field));					\
+	} while (0)
+
 static
 const struct bt_field_class_variant_option *
 borrow_variant_field_selected_class_option(const struct bt_field *field)
@@ -1137,41 +1293,50 @@ borrow_variant_field_selected_class_option(const struct bt_field *field)
 	const struct bt_field_variant *var_field = (const void *) field;
 
 	BT_ASSERT_DBG(field);
-	BT_ASSERT_PRE_DEV(var_field->selected_field,
-		"Variant field has no selected field: %!+f", field);
 	container_fc = (const void *) field->class;
 	return container_fc->named_fcs->pdata[var_field->selected_index];
 }
 
+BT_EXPORT
 const struct bt_field_class_variant_option *
 bt_field_variant_borrow_selected_option_class_const(
 		const struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_CLASS_OPT(field);
 	return borrow_variant_field_selected_class_option(field);
 }
 
+BT_EXPORT
 const struct bt_field_class_variant_with_selector_field_integer_unsigned_option *
 bt_field_variant_with_selector_field_integer_unsigned_borrow_selected_option_class_const(
 		const struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_VARIANT_WITH_UNSIGNED_INTEGER_SELECTOR_FIELD, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"variant-field-with-unsigned-selector-field",
+		BT_FIELD_CLASS_TYPE_VARIANT_WITH_UNSIGNED_INTEGER_SELECTOR_FIELD,
+		"Field");
+	BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_CLASS_OPT(field);
 	return (const void *) borrow_variant_field_selected_class_option(field);
 }
 
+BT_EXPORT
 const struct bt_field_class_variant_with_selector_field_integer_signed_option *
 bt_field_variant_with_selector_field_integer_signed_borrow_selected_option_class_const(
 		const struct bt_field *field)
 {
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE(field,
-		BT_FIELD_CLASS_TYPE_VARIANT_WITH_SIGNED_INTEGER_SELECTOR_FIELD, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_HAS_CLASS_TYPE("field", field,
+		"variant-field-with-signed-selector-field",
+		BT_FIELD_CLASS_TYPE_VARIANT_WITH_SIGNED_INTEGER_SELECTOR_FIELD,
+		"Field");
+	BT_ASSERT_PRE_DEV_FOR_BORROW_VAR_FIELD_SEL_CLASS_OPT(field);
 	return (const void *) borrow_variant_field_selected_class_option(field);
 }
 
+BT_EXPORT
 enum bt_field_variant_select_option_by_index_status
 bt_field_variant_select_option_by_index(
 		struct bt_field *field, uint64_t index)
@@ -1179,27 +1344,103 @@ bt_field_variant_select_option_by_index(
 	struct bt_field_variant *var_field = (void *) field;
 
 	BT_ASSERT_PRE_DEV_NO_ERROR();
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_HOT(field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
 	BT_ASSERT_PRE_DEV_VALID_INDEX(index, var_field->fields->len);
 	var_field->selected_field = var_field->fields->pdata[index];
 	var_field->selected_index = index;
 	return BT_FUNC_STATUS_OK;
 }
 
+BT_EXPORT
 uint64_t bt_field_variant_get_selected_option_index(
 		const struct bt_field *field)
 {
 	const struct bt_field_variant *var_field = (const void *) field;
 
-	BT_ASSERT_PRE_DEV_NON_NULL(field, "Field");
-	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT(field, "Field");
-	BT_ASSERT_PRE_DEV(var_field->selected_field,
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_VARIANT("field", field, "Field");
+	BT_ASSERT_PRE_DEV("has-selected-field", var_field->selected_field,
 		"Variant field has no selected field: %!+f", field);
 	return var_field->selected_index;
 }
 
+BT_EXPORT
+uint8_t *bt_field_blob_get_data(struct bt_field *field)
+{
+	const struct bt_field_blob *blob_field = (const void *) field;
+
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_BLOB("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
+	BT_ASSERT_PRE_DEV("blob-field-length-is-set",
+		blob_field->length > 0,
+		"BLOB field length is not set: %!+f", field);
+
+	/* Assume that the user will fill the bytes. */
+	bt_field_set_single(field, true);
+
+	return blob_field->data;
+}
+
+BT_EXPORT
+const uint8_t *bt_field_blob_get_data_const(const struct bt_field *field)
+{
+	const struct bt_field_blob *blob_field = (const void *) field;
+
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_BLOB("field", field, "Field");
+
+	return blob_field->data;
+}
+
+BT_EXPORT
+enum bt_field_blob_dynamic_set_length_status bt_field_blob_dynamic_set_length(
+		struct bt_field *field, uint64_t length)
+{
+	int ret;
+	struct bt_field_blob *blob_field = (void *) field;
+
+	BT_ASSERT_PRE_DEV_NO_ERROR();
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_DYNAMIC_BLOB("field", field, "Field");
+	BT_ASSERT_PRE_DEV_FIELD_HOT(field);
+
+	if (G_UNLIKELY(length > blob_field->length)) {
+		/* Make more room */
+		uint8_t *data = g_realloc(blob_field->data, length);
+
+		if (!data) {
+			BT_LIB_LOGE_APPEND_CAUSE(
+				"Failed to reallocate BLOB field data: %!+f",
+				field);
+			ret = BT_FIELD_DYNAMIC_BLOB_SET_LENGTH_STATUS_MEMORY_ERROR;
+			goto end;
+		}
+
+		blob_field->data = data;
+	}
+
+	blob_field->length = length;
+	ret = BT_FIELD_DYNAMIC_BLOB_SET_LENGTH_STATUS_OK;
+
+end:
+	return ret;
+}
+
+BT_EXPORT
+uint64_t bt_field_blob_get_length(const struct bt_field *field)
+{
+	const struct bt_field_blob *blob_field = (const void *) field;
+
+	BT_ASSERT_PRE_DEV_FIELD_NON_NULL(field);
+	BT_ASSERT_PRE_DEV_FIELD_IS_BLOB("field", field, "Field");
+
+	return blob_field->length;
+}
+
+
 static inline
 void bt_field_finalize(struct bt_field *field)
 {
@@ -1294,6 +1535,20 @@ void destroy_variant_field(struct bt_field *field)
 	g_free(field);
 }
 
+static
+void destroy_blob_field(struct bt_field *field)
+{
+	struct bt_field_blob *blob_field = (void *) field;
+
+	BT_ASSERT(field);
+	BT_LIB_LOGD("Destroying BLOB field object: %!+f", field);
+	bt_field_finalize(field);
+
+	g_free(blob_field->data);
+
+	g_free(field);
+}
+
 static
 void destroy_array_field(struct bt_field *field)
 {
@@ -1328,7 +1583,6 @@ void destroy_string_field(struct bt_field *field)
 	g_free(field);
 }
 
-BT_HIDDEN
 void bt_field_destroy(struct bt_field *field)
 {
 	BT_ASSERT(field);
@@ -1372,6 +1626,11 @@ void bt_field_destroy(struct bt_field *field)
 	case BT_FIELD_CLASS_TYPE_VARIANT_WITH_SIGNED_INTEGER_SELECTOR_FIELD:
 		destroy_variant_field(field);
 		break;
+	case BT_FIELD_CLASS_TYPE_STATIC_BLOB:
+	case BT_FIELD_CLASS_TYPE_DYNAMIC_BLOB_WITHOUT_LENGTH_FIELD:
+	case BT_FIELD_CLASS_TYPE_DYNAMIC_BLOB_WITH_LENGTH_FIELD:
+		destroy_blob_field(field);
+		break;
 	default:
 		bt_common_abort();
 	}
@@ -1513,7 +1772,6 @@ void set_array_field_is_frozen(struct bt_field *field, bool is_frozen)
 	set_single_field_is_frozen(field, is_frozen);
 }
 
-BT_HIDDEN
 void _bt_field_set_is_frozen(const struct bt_field *field,
 		bool is_frozen)
 {
