@@ -1,26 +1,8 @@
 /*
+ * SPDX-License-Identifier: MIT
+ *
  * Copyright 2016 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  * Copyright 2016 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- *
- * Author: Jérémie Galarneau <jeremie.galarneau@efficios.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
  */
 
 #include <babeltrace2/babeltrace.h>
@@ -639,7 +621,7 @@ void print_escape_string(struct pretty_component *pretty, const char *str)
 		}
 
 		/* Standard characters. */
-		if (!iscntrl(str[i])) {
+		if (!iscntrl((unsigned char) str[i])) {
 			bt_common_g_string_append_c(pretty->string, str[i]);
 			continue;
 		}
@@ -682,21 +664,308 @@ void print_escape_string(struct pretty_component *pretty, const char *str)
 	bt_common_g_string_append_c(pretty->string, '"');
 }
 
+/*
+ * Print the unknown label.
+ */
+static
+void print_enum_value_label_unknown(struct pretty_component *pretty)
+{
+	if (pretty->use_colors) {
+		bt_common_g_string_append(pretty->string, color_unknown);
+	}
+
+	bt_common_g_string_append(pretty->string, "<unknown>");
+
+	if (pretty->use_colors) {
+		bt_common_g_string_append(pretty->string, color_rst);
+	}
+}
+
+/*
+ * Print labels for a value. If there are more than one label for the
+ * value, they will be printed in "{ }".
+ */
+static
+void print_enum_value_label_array(struct pretty_component *pretty,
+		uint64_t label_count,
+		bt_field_class_enumeration_mapping_label_array label_array)
+{
+	uint64_t i;
+
+	if (label_count > 1) {
+		bt_common_g_string_append(pretty->string, "{ ");
+	}
+
+	for (i = 0; i < label_count; i++) {
+		const char *mapping_name = label_array[i];
+
+		if (i != 0) {
+			bt_common_g_string_append(pretty->string, ", ");
+		}
+		if (pretty->use_colors) {
+			bt_common_g_string_append(pretty->string, color_enum_mapping_name);
+		}
+		print_escape_string(pretty, mapping_name);
+		if (pretty->use_colors) {
+			bt_common_g_string_append(pretty->string, color_rst);
+		}
+	}
+
+	if (label_count > 1) {
+		bt_common_g_string_append(pretty->string, " }");
+	}
+}
+
+/*
+ * Print arrays of labels and counts are ORed bit flags.
+ */
+static
+void print_enum_value_bit_flag_label_arrays(struct pretty_component *pretty)
+{
+	uint64_t i;
+	bool first_label = true;
+
+	/* For each bit with a label count > 0, print the labels. */
+	for (i = 0; i < ENUMERATION_MAX_BITFLAGS_COUNT; i++) {
+		uint64_t label_count = pretty->enum_bit_labels[i]->len;
+
+		if (label_count > 0) {
+			if (!first_label) {
+				bt_common_g_string_append(pretty->string, " | ");
+			}
+			print_enum_value_label_array(pretty, label_count,
+				(void *) pretty->enum_bit_labels[i]->pdata);
+			first_label = false;
+		}
+	}
+}
+
+/*
+ * Get the labels mapping to an unsigned value.
+ *
+ * This function will set the count to the count of mapping labels that match
+ * the value. If count == 0, the caller has nothing to do, if count > 0,
+ * the label_array contains the labels to print and the caller is responsible
+ * to call g_free on it once the values have been used.
+ */
+static
+void print_enum_unsigned_get_mapping_labels_for_value(const bt_field_class *fc,
+		uint64_t value, GPtrArray *labels)
+{
+	uint64_t mapping_count = bt_field_class_enumeration_get_mapping_count(fc);
+	uint64_t i;
+
+	for (i = 0; i < mapping_count; i++) {
+		uint64_t range_i;
+		const struct bt_field_class_enumeration_unsigned_mapping *mapping =
+			bt_field_class_enumeration_unsigned_borrow_mapping_by_index_const(fc, i);
+		const bt_integer_range_set_unsigned *ranges =
+			bt_field_class_enumeration_unsigned_mapping_borrow_ranges_const(mapping);
+		uint64_t range_count = bt_integer_range_set_get_range_count(
+				bt_integer_range_set_unsigned_as_range_set_const(ranges));
+
+		for (range_i = 0; range_i < range_count; range_i++) {
+			const bt_integer_range_unsigned *range =
+				bt_integer_range_set_unsigned_borrow_range_by_index_const(ranges, range_i);
+			uint64_t lower = bt_integer_range_unsigned_get_lower(range);
+			uint64_t upper = bt_integer_range_unsigned_get_upper(range);
+
+			/*
+			 * Flag is active if this range represents a single value
+			 * (lower == upper) and the lower is the same as the bit
+			 * value to test against.
+			 */
+			if ((lower == upper) && (lower == value)) {
+				g_ptr_array_add(labels, (void *) bt_field_class_enumeration_mapping_get_label(
+					bt_field_class_enumeration_unsigned_mapping_as_mapping_const(mapping)));
+				break;
+			}
+		}
+	}
+}
+
+/*
+ * Splits an unsigned enum value into its bit and for each bit set,
+ * try to find a corresponding label.
+ *
+ * If any bit set does not have a corresponding label, then it prints
+ * an unknown value, otherwise, it prints the labels, separated by '|'
+ */
+static
+void print_enum_unsigned_try_bit_flags(struct pretty_component *pretty,
+		const bt_field *field)
+{
+	uint64_t i;
+	uint64_t value = bt_field_integer_unsigned_get_value(field);
+	const bt_field_class *fc = bt_field_borrow_class_const(field);
+
+	/* Value is 0, if there was a label for it, we would know by now. */
+	if (value == 0) {
+		print_enum_value_label_unknown(pretty);
+		goto end;
+	}
+
+	for (i = 0; i < ENUMERATION_MAX_BITFLAGS_COUNT; i++) {
+		uint64_t bit_value = UINT64_C(1) << i;
+
+		if ((value & bit_value) != 0) {
+			print_enum_unsigned_get_mapping_labels_for_value(
+				fc, bit_value, pretty->enum_bit_labels[i]);
+
+			if (pretty->enum_bit_labels[i]->len == 0) {
+				/*
+				 * This bit has no matching label, so this
+				 * field is not a bit flag field, print
+				 * unknown and return.
+				 */
+				print_enum_value_label_unknown(pretty);
+				goto end;
+			}
+		}
+	}
+
+	print_enum_value_bit_flag_label_arrays(pretty);
+
+end:
+	return;
+}
+
+/*
+ * Get the labels mapping to a signed value
+ *
+ * This function will set the count to the count of mapping labels that match
+ * the value. If count == 0, the caller has nothing to do, if count > 0,
+ * the label_array contains the labels to print and the caller is responsible
+ * to call g_free on it once the values have been used.
+ */
+static
+void print_enum_signed_get_mapping_labels_for_value(const bt_field_class *fc,
+		int64_t value, GPtrArray *labels)
+{
+	uint64_t mapping_count = bt_field_class_enumeration_get_mapping_count(fc);
+	uint64_t i;
+
+	for (i = 0; i < mapping_count; i++) {
+		uint64_t range_i;
+		const struct bt_field_class_enumeration_signed_mapping *mapping =
+			bt_field_class_enumeration_signed_borrow_mapping_by_index_const(fc, i);
+		const bt_integer_range_set_signed *ranges =
+			bt_field_class_enumeration_signed_mapping_borrow_ranges_const(mapping);
+		uint64_t range_count = bt_integer_range_set_get_range_count(
+				bt_integer_range_set_signed_as_range_set_const(ranges));
+
+		for (range_i = 0; range_i < range_count; range_i++) {
+			const bt_integer_range_signed *range =
+				bt_integer_range_set_signed_borrow_range_by_index_const(ranges, range_i);
+			uint64_t lower = bt_integer_range_signed_get_lower(range);
+			uint64_t upper = bt_integer_range_signed_get_upper(range);
+
+			/*
+			 * Flag is active if this range represents a single value
+			 * (lower == upper) and the lower is the same as the bit
+			 * value to test against.
+			 */
+			if ((lower == upper) && (lower == value)) {
+				g_ptr_array_add(labels, (void*) bt_field_class_enumeration_mapping_get_label(
+					bt_field_class_enumeration_signed_mapping_as_mapping_const(mapping)));
+				break;
+			}
+		}
+	}
+}
+
+/*
+ * Splits a signed enum value into its bit and for each bit set,
+ * try to find a corresponding label.
+ *
+ * If any bit set does not have a corresponding label, then it prints
+ * an unknown value, otherwise, it prints the labels, separated by '|'.
+ */
+static
+void print_enum_signed_try_bit_flags(struct pretty_component *pretty,
+		const bt_field *field)
+{
+	uint64_t i;
+	int64_t value = bt_field_integer_signed_get_value(field);
+	const bt_field_class *fc = bt_field_borrow_class_const(field);
+
+	/*
+	 * Negative value, not a bit flag enum
+	 * For 0, if there was a value, we would know by now.
+	 */
+	if (value <= 0) {
+		print_enum_value_label_unknown(pretty);
+		goto end;
+	}
+
+	for (i = 0; i < ENUMERATION_MAX_BITFLAGS_COUNT; i++) {
+		uint64_t bit_value = 1ULL << i;
+
+		if ((value & bit_value) != 0) {
+			print_enum_signed_get_mapping_labels_for_value(
+				fc, bit_value, pretty->enum_bit_labels[i]);
+
+			if (pretty->enum_bit_labels[i]->len == 0) {
+				/*
+				 * This bit has no matching label, so this
+				 * field is not a bit flag field, print
+				 * unknown and return.
+				 */
+				print_enum_value_label_unknown(pretty);
+				goto end;
+			}
+		}
+	}
+
+	print_enum_value_bit_flag_label_arrays(pretty);
+
+end:
+	return;
+}
+
+/*
+ * Main function to try to print the value of the enum field as a
+ * bit flag. It calls the appropriate function depending on the
+ * field type's signedness and prints the label for each bit set,
+ * if it's a bit flag field.
+ */
+static
+void print_enum_try_bit_flags(struct pretty_component *pretty,
+		const bt_field *field)
+{
+	const bt_field_class *fc = bt_field_borrow_class_const(field);
+	uint64_t int_range = bt_field_class_integer_get_field_value_range(fc);
+	int i;
+
+	BT_ASSERT(int_range <= ENUMERATION_MAX_BITFLAGS_COUNT);
+
+	/*
+	 * Remove all labels from the previous enumeration field.
+	 */
+	for (i = 0; i < ENUMERATION_MAX_BITFLAGS_COUNT; i++) {
+		g_ptr_array_set_size(pretty->enum_bit_labels[i], 0);
+	}
+
+	/* Get the mapping labels for the bit value. */
+	switch (bt_field_class_get_type(fc)) {
+	case BT_FIELD_CLASS_TYPE_UNSIGNED_ENUMERATION:
+		print_enum_unsigned_try_bit_flags(pretty, field);
+		break;
+	case BT_FIELD_CLASS_TYPE_SIGNED_ENUMERATION:
+		print_enum_signed_try_bit_flags(pretty, field);
+		break;
+	default:
+		bt_common_abort();
+	}
+}
+
 static
 int print_enum(struct pretty_component *pretty,
 		const bt_field *field)
 {
 	int ret = 0;
-	const bt_field_class *enumeration_field_class = NULL;
 	bt_field_class_enumeration_mapping_label_array label_array;
 	uint64_t label_count;
-	uint64_t i;
-
-	enumeration_field_class = bt_field_borrow_class_const(field);
-	if (!enumeration_field_class) {
-		ret = -1;
-		goto end;
-	}
 
 	switch (bt_field_get_class_type(field)) {
 	case BT_FIELD_CLASS_TYPE_UNSIGNED_ENUMERATION:
@@ -717,31 +986,22 @@ int print_enum(struct pretty_component *pretty,
 	}
 
 	bt_common_g_string_append(pretty->string, "( ");
-	if (label_count == 0) {
-		if (pretty->use_colors) {
-			bt_common_g_string_append(pretty->string, color_unknown);
-		}
-		bt_common_g_string_append(pretty->string, "<unknown>");
-		if (pretty->use_colors) {
-			bt_common_g_string_append(pretty->string, color_rst);
-		}
-		goto skip_loop;
+	if (label_count != 0) {
+		/* The numerical value matches some labels, print them. */
+		print_enum_value_label_array(pretty, label_count, label_array);
+	} else if (pretty->options.print_enum_flags) {
+		/*
+		 * The numerical value of the enumeration does not match any
+		 * label but the `print-enum-flags` parameter is TRUE so try to
+		 * decompose the value of the enum into bits and print it as a
+		 * bit flag enum.
+		 */
+		print_enum_try_bit_flags(pretty, field);
+	} else {
+		print_enum_value_label_unknown(pretty);
 	}
-	for (i = 0; i < label_count; i++) {
-		const char *mapping_name = label_array[i];
 
-		if (i != 0) {
-			bt_common_g_string_append(pretty->string, ", ");
-		}
-		if (pretty->use_colors) {
-			bt_common_g_string_append(pretty->string, color_enum_mapping_name);
-		}
-		print_escape_string(pretty, mapping_name);
-		if (pretty->use_colors) {
-			bt_common_g_string_append(pretty->string, color_rst);
-		}
-	}
-skip_loop:
+	/* Print the actual value of the enum. */
 	bt_common_g_string_append(pretty->string, " : container = ");
 	ret = print_integer(pretty, field);
 	if (ret != 0) {
@@ -752,6 +1012,83 @@ end:
 	return ret;
 }
 
+static
+gint compare_strings(gconstpointer a, gconstpointer b)
+{
+    return strcmp(*(const char **) a, *(const char **) b);
+}
+
+static
+int print_bit_array(struct pretty_component *pretty, const bt_field *field)
+{
+	int ret = 0;
+	bt_field_class_bit_array_flag_label_array label_array;
+	uint64_t label_count;
+	uint64_t v = bt_field_bit_array_get_value_as_integer(field);
+	GArray *g_label_array = NULL;
+
+	if (pretty->use_colors) {
+		bt_common_g_string_append(pretty->string,
+			color_number_value);
+	}
+
+	bt_common_g_string_append_printf(pretty->string, "0x%" PRIX64, v);
+
+	if (pretty->use_colors) {
+		bt_common_g_string_append(pretty->string, color_rst);
+	}
+
+	ret = bt_field_bit_array_get_active_flag_labels(field, &label_array, &label_count);
+	if (ret) {
+		ret = -1;
+		goto end;
+	}
+
+	if (label_count > 0) {
+		uint64_t i;
+
+		g_label_array = g_array_new(FALSE, FALSE, sizeof(const char *));
+		if (!g_label_array) {
+			ret = -1;
+			goto end;
+		}
+
+		for (i = 0; i < label_count; i++) {
+			g_array_append_val(g_label_array, label_array[i]);
+		}
+
+		g_array_sort(g_label_array, compare_strings);
+		bt_common_g_string_append(pretty->string, " { ");
+
+		for (i = 0; i < g_label_array->len; i++) {
+			const char *flag_name = g_array_index(g_label_array, const char *, i);
+
+			if (i != 0) {
+				bt_common_g_string_append(pretty->string, ", ");
+			}
+
+			if (pretty->use_colors) {
+				bt_common_g_string_append(pretty->string, color_enum_mapping_name);
+			}
+
+			print_escape_string(pretty, flag_name);
+
+			if (pretty->use_colors) {
+				bt_common_g_string_append(pretty->string, color_rst);
+			}
+		}
+
+		bt_common_g_string_append(pretty->string, " }");
+	}
+
+end:
+	if (g_label_array) {
+		g_array_free(g_label_array, TRUE);
+	}
+
+	return ret;
+}
+
 static
 int print_struct_field(struct pretty_component *pretty,
 		const bt_field *_struct,
@@ -797,11 +1134,6 @@ int print_struct(struct pretty_component *pretty,
 	uint64_t nr_fields, i, nr_printed_fields;
 
 	struct_class = bt_field_borrow_class_const(_struct);
-	if (!struct_class) {
-		ret = -1;
-		goto end;
-	}
-
 	nr_fields = bt_field_class_structure_get_member_count(struct_class);
 
 	bt_common_g_string_append(pretty->string, "{");
@@ -846,15 +1178,9 @@ int print_array(struct pretty_component *pretty,
 		const bt_field *array, bool print_names)
 {
 	int ret = 0;
-	const bt_field_class *array_class = NULL;
 	uint64_t len;
 	uint64_t i;
 
-	array_class = bt_field_borrow_class_const(array);
-	if (!array_class) {
-		ret = -1;
-		goto end;
-	}
 	len = bt_field_array_get_length(array);
 	bt_common_g_string_append(pretty->string, "[");
 	pretty->depth++;
@@ -971,6 +1297,25 @@ end:
 	return ret;
 }
 
+static
+int print_blob(struct pretty_component *pretty, const bt_field *blob)
+{
+	const uint8_t *data = bt_field_blob_get_data_const(blob);
+	uint64_t len = bt_field_blob_get_length(blob);
+	size_t i;
+
+	bt_common_g_string_append(pretty->string, "{ ");
+
+	for (i = 0; i < len; ++i) {
+		uint8_t b = data[i];
+
+		bt_common_g_string_append_printf(pretty->string, "%02x ", b);
+	}
+
+	bt_common_g_string_append(pretty->string, "}");
+	return 0;
+}
+
 static
 int print_field(struct pretty_component *pretty,
 		const bt_field *field, bool print_names)
@@ -997,18 +1342,7 @@ int print_field(struct pretty_component *pretty,
 		}
 		return 0;
 	} else if (class_id == BT_FIELD_CLASS_TYPE_BIT_ARRAY) {
-		uint64_t v = bt_field_bit_array_get_value_as_integer(field);
-
-		if (pretty->use_colors) {
-			bt_common_g_string_append(pretty->string,
-				color_number_value);
-		}
-		bt_common_g_string_append_printf(pretty->string, "0x%" PRIX64,
-			v);
-		if (pretty->use_colors) {
-			bt_common_g_string_append(pretty->string, color_rst);
-		}
-		return 0;
+		return print_bit_array(pretty, field);
 	} else if (bt_field_class_type_is(class_id,
 			BT_FIELD_CLASS_TYPE_ENUMERATION)) {
 		return print_enum(pretty, field);
@@ -1062,11 +1396,11 @@ int print_field(struct pretty_component *pretty,
 	} else if (bt_field_class_type_is(class_id,
 			BT_FIELD_CLASS_TYPE_DYNAMIC_ARRAY)) {
 		return print_sequence(pretty, field, print_names);
-	} else {
-		// TODO: log instead
-		fprintf(pretty->err, "[error] Unknown type id: %d\n", (int) class_id);
-		return -1;
+	} else if (bt_field_class_type_is(class_id, BT_FIELD_CLASS_TYPE_BLOB)) {
+		return print_blob(pretty, field);
 	}
+
+	bt_common_abort();
 }
 
 static
@@ -1191,7 +1525,6 @@ end:
 	return ret;
 }
 
-BT_HIDDEN
 int pretty_print_event(struct pretty_component *pretty,
 		const bt_message *event_msg)
 {
@@ -1250,6 +1583,7 @@ int print_discarded_elements_msg(struct pretty_component *pretty,
 	const char *stream_name;
 	const char *trace_name;
 	bt_uuid trace_uuid;
+	const char *trace_uid;
 	int64_t stream_class_id;
 	int64_t stream_id;
 	const char *init_msg;
@@ -1276,8 +1610,12 @@ int print_discarded_elements_msg(struct pretty_component *pretty,
 		trace_name = "(unknown)";
 	}
 
-	/* Trace UUID */
-	trace_uuid = bt_trace_get_uuid(trace);
+	/* Trace UUID or UID */
+	if (pretty->mip_version == 0) {
+		trace_uuid = bt_trace_get_uuid(trace);
+	} else {
+		trace_uid = bt_trace_get_uid(trace);
+	}
 
 	/* Format message */
 	g_string_assign(pretty->string, "");
@@ -1317,12 +1655,22 @@ int print_discarded_elements_msg(struct pretty_component *pretty,
 
 	bt_common_g_string_append_printf(pretty->string, " in trace \"%s\" ", trace_name);
 
-	if (trace_uuid) {
-		bt_common_g_string_append_printf(pretty->string,
-			"(UUID: " BT_UUID_FMT ") ",
-			BT_UUID_FMT_VALUES(trace_uuid));
+	if (pretty->mip_version == 0) {
+		if (trace_uuid) {
+			bt_common_g_string_append_printf(pretty->string,
+				"(UUID: " BT_UUID_FMT ") ",
+				BT_UUID_FMT_VALUES(trace_uuid));
+		} else {
+			bt_common_g_string_append(pretty->string, "(no UUID) ");
+		}
 	} else {
-		bt_common_g_string_append(pretty->string, "(no UUID) ");
+		if (trace_uid) {
+			bt_common_g_string_append_printf(pretty->string,
+				"(UID: %s) ", trace_uid);
+		} else {
+			bt_common_g_string_append(pretty->string, "(no UID) ");
+
+		}
 	}
 
 	bt_common_g_string_append_printf(pretty->string,
@@ -1350,7 +1698,6 @@ int print_discarded_elements_msg(struct pretty_component *pretty,
 	return ret;
 }
 
-BT_HIDDEN
 int pretty_print_discarded_items(struct pretty_component *pretty,
 		const bt_message *msg)
 {
@@ -1419,7 +1766,6 @@ int pretty_print_discarded_items(struct pretty_component *pretty,
 	return 0;
 }
 
-BT_HIDDEN
 void pretty_print_init(void)
 {
 	strcpy(color_name, bt_common_color_bold());
