@@ -1,24 +1,8 @@
 /*
+ * SPDX-License-Identifier: MIT
+ *
  * Copyright 2016 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  * Copyright 2019 Philippe Proulx <pproulx@efficios.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
  */
 
 #define BT_LOG_OUTPUT_LEVEL (trimmer_comp->log_level)
@@ -143,6 +127,17 @@ struct trimmer_iterator_stream_state {
 	const bt_packet *cur_packet;
 };
 
+bt_component_class_get_supported_mip_versions_method_status
+trimmer_supported_mip_versions(
+		bt_self_component_class_filter *self_component_class __attribute__((unused)),
+		const bt_value *params __attribute__((unused)),
+		void *initialize_method_data __attribute__((unused)),
+		bt_logging_level logging_level __attribute__((unused)),
+		bt_integer_range_set_unsigned *supported_versions)
+{
+	return (int) bt_integer_range_set_unsigned_add_range(supported_versions, 0, 1);
+}
+
 static
 void destroy_trimmer_comp(struct trimmer_comp *trimmer_comp)
 {
@@ -156,7 +151,6 @@ struct trimmer_comp *create_trimmer_comp(void)
 	return g_new0(struct trimmer_comp, 1);
 }
 
-BT_HIDDEN
 void trimmer_finalize(bt_self_component_filter *self_comp)
 {
 	struct trimmer_comp *trimmer_comp =
@@ -190,7 +184,7 @@ bool compile_and_match(const char *pattern, const char *string, GMatchInfo **mat
 		/*
 		 * g_regex_match allocates `*match_info` even if it returns
 		 * FALSE.  If there's no match, we have no use for it, so free
-		 * it immediatly and don't return it to the caller.
+		 * it immediately and don't return it to the caller.
 		 */
 		g_match_info_free(*match_info);
 		*match_info = NULL;
@@ -442,7 +436,7 @@ end:
  */
 static
 int set_bound_from_param(struct trimmer_comp *trimmer_comp,
-		const char *param_name, const bt_value *param,
+		const bt_value *param,
 		struct trimmer_bound *bound, bool is_gmt)
 {
 	int ret;
@@ -456,7 +450,7 @@ int set_bound_from_param(struct trimmer_comp *trimmer_comp,
 		 * Just convert it to a temporary string to handle
 		 * everything the same way.
 		 */
-		sprintf(tmp_arg, "%" PRId64, value);
+		snprintf(tmp_arg, sizeof(tmp_arg), "%" PRId64, value);
 		arg = tmp_arg;
 	} else {
 		BT_ASSERT(bt_value_is_string(param));
@@ -568,7 +562,7 @@ bt_component_class_initialize_method_status init_trimmer_comp_from_params(
 
         value = bt_value_map_borrow_entry_value_const(params, "begin");
 	if (value) {
-		if (set_bound_from_param(trimmer_comp, "begin", value,
+		if (set_bound_from_param(trimmer_comp, value,
 				&trimmer_comp->begin, trimmer_comp->is_gmt)) {
 			/* set_bound_from_param() logs errors */
 			status = BT_COMPONENT_CLASS_INITIALIZE_METHOD_STATUS_ERROR;
@@ -581,7 +575,7 @@ bt_component_class_initialize_method_status init_trimmer_comp_from_params(
 
         value = bt_value_map_borrow_entry_value_const(params, "end");
 	if (value) {
-		if (set_bound_from_param(trimmer_comp, "end", value,
+		if (set_bound_from_param(trimmer_comp, value,
 				&trimmer_comp->end, trimmer_comp->is_gmt)) {
 			/* set_bound_from_param() logs errors */
 			status = BT_COMPONENT_CLASS_INITIALIZE_METHOD_STATUS_ERROR;
@@ -611,8 +605,9 @@ end:
 
 bt_component_class_initialize_method_status trimmer_init(
 		bt_self_component_filter *self_comp_flt,
-		bt_self_component_filter_configuration *config,
-		const bt_value *params, void *init_data)
+		bt_self_component_filter_configuration *config __attribute__((unused)),
+		const bt_value *params,
+		void *init_data __attribute__((unused)))
 {
 	bt_component_class_initialize_method_status status;
 	bt_self_component_add_port_status add_port_status;
@@ -695,11 +690,10 @@ void destroy_trimmer_iterator_stream_state(
 	g_free(sstate);
 }
 
-BT_HIDDEN
 bt_message_iterator_class_initialize_method_status trimmer_msg_iter_init(
 		bt_self_message_iterator *self_msg_iter,
 		bt_self_message_iterator_configuration *config,
-		bt_self_component_port_output *port)
+		bt_self_component_port_output *port __attribute__((unused)))
 {
 	bt_message_iterator_class_initialize_method_status status;
 	bt_message_iterator_create_from_message_iterator_status
@@ -800,27 +794,33 @@ int get_msg_ns_from_origin(const bt_message *msg, int64_t *ns_from_origin,
 			msg);
 		break;
 	case BT_MESSAGE_TYPE_PACKET_BEGINNING:
-		clock_class =
-			bt_message_packet_beginning_borrow_stream_class_default_clock_class_const(
-				msg);
-		if (G_UNLIKELY(!clock_class)) {
-			goto error;
+	{
+		const bt_packet *packet = bt_message_packet_beginning_borrow_packet_const(msg);
+		const bt_stream *stream = bt_packet_borrow_stream_const(packet);
+		const bt_stream_class *stream_class = bt_stream_borrow_class_const(stream);
+
+		if (!bt_stream_class_packets_have_beginning_default_clock_snapshot(stream_class)) {
+			goto no_clock_snapshot;
 		}
 
 		clock_snapshot = bt_message_packet_beginning_borrow_default_clock_snapshot_const(
 			msg);
 		break;
+	}
 	case BT_MESSAGE_TYPE_PACKET_END:
-		clock_class =
-			bt_message_packet_end_borrow_stream_class_default_clock_class_const(
-				msg);
-		if (G_UNLIKELY(!clock_class)) {
-			goto error;
+	{
+		const bt_packet *packet = bt_message_packet_end_borrow_packet_const(msg);
+		const bt_stream *stream = bt_packet_borrow_stream_const(packet);
+		const bt_stream_class *stream_class = bt_stream_borrow_class_const(stream);
+
+		if (!bt_stream_class_packets_have_end_default_clock_snapshot(stream_class)) {
+			goto no_clock_snapshot;
 		}
 
 		clock_snapshot = bt_message_packet_end_borrow_default_clock_snapshot_const(
 			msg);
 		break;
+	}
 	case BT_MESSAGE_TYPE_STREAM_BEGINNING:
 	{
 		enum bt_message_stream_clock_snapshot_state cs_state;
@@ -856,27 +856,31 @@ int get_msg_ns_from_origin(const bt_message *msg, int64_t *ns_from_origin,
 		break;
 	}
 	case BT_MESSAGE_TYPE_DISCARDED_EVENTS:
-		clock_class =
-			bt_message_discarded_events_borrow_stream_class_default_clock_class_const(
-				msg);
-		if (G_UNLIKELY(!clock_class)) {
-			goto error;
+	{
+		const bt_stream *stream = bt_message_discarded_events_borrow_stream_const(msg);
+		const bt_stream_class *stream_class = bt_stream_borrow_class_const(stream);
+
+		if (!bt_stream_class_discarded_events_have_default_clock_snapshots(stream_class)) {
+			goto no_clock_snapshot;
 		}
 
 		clock_snapshot = bt_message_discarded_events_borrow_beginning_default_clock_snapshot_const(
 			msg);
 		break;
+	}
 	case BT_MESSAGE_TYPE_DISCARDED_PACKETS:
-		clock_class =
-			bt_message_discarded_packets_borrow_stream_class_default_clock_class_const(
-				msg);
-		if (G_UNLIKELY(!clock_class)) {
-			goto error;
+	{
+		const bt_stream *stream = bt_message_discarded_packets_borrow_stream_const(msg);
+		const bt_stream_class *stream_class = bt_stream_borrow_class_const(stream);
+
+		if (!bt_stream_class_discarded_packets_have_default_clock_snapshots(stream_class)) {
+			goto no_clock_snapshot;
 		}
 
 		clock_snapshot = bt_message_discarded_packets_borrow_beginning_default_clock_snapshot_const(
 			msg);
 		break;
+	}
 	case BT_MESSAGE_TYPE_MESSAGE_ITERATOR_INACTIVITY:
 		clock_snapshot =
 			bt_message_message_iterator_inactivity_borrow_clock_snapshot_const(
@@ -1881,7 +1885,6 @@ end:
 	return status;
 }
 
-BT_HIDDEN
 bt_message_iterator_class_next_method_status trimmer_msg_iter_next(
 		bt_self_message_iterator *self_msg_iter,
 		bt_message_array_const msgs, uint64_t capacity,
@@ -1950,7 +1953,6 @@ end:
 	return status;
 }
 
-BT_HIDDEN
 void trimmer_msg_iter_finalize(bt_self_message_iterator *self_msg_iter)
 {
 	struct trimmer_iterator *trimmer_it =
