@@ -1,27 +1,9 @@
 /*
- * python-plugin-provider.c
- *
- * Babeltrace Python plugin provider
+ * SPDX-License-Identifier: MIT
  *
  * Copyright 2017 Philippe Proulx <pproulx@efficios.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Babeltrace Python plugin provider
  */
 
 #define BT_LOG_TAG "LIB/PLUGIN-PY"
@@ -42,7 +24,6 @@
 #include <signal.h>
 #include <Python.h>
 #include <glib.h>
-#include <gmodule.h>
 
 #define PYTHON_PLUGIN_FILE_PREFIX	"bt_plugin_"
 #define PYTHON_PLUGIN_FILE_PREFIX_LEN	(sizeof(PYTHON_PLUGIN_FILE_PREFIX) - 1)
@@ -103,7 +84,7 @@ void log_python_traceback(int log_level)
 			goto end;
 		}
 
-		BT_LOG_WRITE(log_level, BT_LOG_TAG,
+		BT_LOG_WRITE_PRINTF(log_level, BT_LOG_TAG,
 			"Exception occurred: Python traceback:\n%s", exc->str);
 	}
 
@@ -589,7 +570,7 @@ int bt_plugin_from_python_plugin_info(PyObject *plugin_info,
 					bt_plugin_get_name(*plugin_out),
 					comp_class,
 					bt_component_class_get_name(comp_class),
-					bt_component_class_type_string(
+					bt_common_component_class_type_string(
 						bt_component_class_get_type(comp_class)));
 				goto error;
 			}
@@ -614,7 +595,7 @@ end:
 	return status;
 }
 
-G_MODULE_EXPORT
+BT_EXPORT
 int bt_plugin_python_create_all_from_file(const char *path,
 		bool fail_on_load_error, struct bt_plugin_set **plugin_set_out)
 {
@@ -682,7 +663,7 @@ int bt_plugin_python_create_all_from_file(const char *path,
 	/*
 	 * Initialize Python now.
 	 *
-	 * This is not done in the library contructor because the
+	 * This is not done in the library constructor because the
 	 * interpreter is somewhat slow to initialize. If you don't
 	 * have any potential Python plugins, you don't need to endure
 	 * this waiting time everytime you load the library.
