@@ -1,34 +1,22 @@
+# SPDX-License-Identifier: GPL-2.0-only
 #
 # Copyright (C) 2019 EfficiOS Inc.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; only version 2
-# of the License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-#
 
 import unittest
+
 import bt2
 import utils
-from utils import TestOutputPortMessageIterator
-from bt2 import clock_snapshot as bt2_clock_snapshot
 from bt2 import event as bt2_event
-from bt2 import event_class as bt2_event_class
 from bt2 import field as bt2_field
+from bt2 import trace as bt2_trace
 from bt2 import packet as bt2_packet
 from bt2 import stream as bt2_stream
-from bt2 import stream_class as bt2_stream_class
-from bt2 import trace as bt2_trace
+from bt2 import event_class as bt2_event_class
 from bt2 import trace_class as bt2_trace_class
+from bt2 import stream_class as bt2_stream_class
+from bt2 import clock_snapshot as bt2_clock_snapshot
+from utils import TestOutputPortMessageIterator
 
 
 class AllMessagesTestCase(unittest.TestCase):
@@ -37,7 +25,7 @@ class AllMessagesTestCase(unittest.TestCase):
             def __init__(self, config, self_port_output):
                 self._at = 0
                 self._with_stream_msgs_clock_snapshots = self_port_output.user_data.get(
-                    'with_stream_msgs_clock_snapshots', False
+                    "with_stream_msgs_clock_snapshots", False
                 )
 
             def __next__(self):
@@ -120,10 +108,11 @@ class AllMessagesTestCase(unittest.TestCase):
 
         class MySrc(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out', params)
+                self._add_output_port("out", params)
 
-                with_cc = bool(params['with_cc'])
+                with_cc = bool(params["with_cc"])
                 tc = self._create_trace_class()
+
                 if with_cc:
                     cc = self._create_clock_class()
                 else:
@@ -140,30 +129,21 @@ class AllMessagesTestCase(unittest.TestCase):
                     discarded_packets_have_default_clock_snapshots=with_cc,
                 )
 
-                # Create payload field class
-                my_int_fc = tc.create_signed_integer_field_class(32)
-                payload_fc = tc.create_structure_field_class()
-                payload_fc += [('my_int', my_int_fc)]
-
-                # Create specific context field class
-                my_int_fc = tc.create_signed_integer_field_class(32)
-                specific_fc = tc.create_structure_field_class()
-                specific_fc += [('my_int', my_int_fc)]
-
-                ec = sc.create_event_class(
-                    name='salut',
-                    payload_field_class=payload_fc,
-                    specific_context_field_class=specific_fc,
-                )
-
                 trace = tc()
                 stream = trace.create_stream(sc)
-                packet = stream.create_packet()
 
                 test_obj._trace = trace
                 test_obj._stream = stream
-                test_obj._packet = packet
-                test_obj._event_class = ec
+                test_obj._packet = stream.create_packet()
+                test_obj._event_class = sc.create_event_class(
+                    name="salut",
+                    payload_field_class=tc.create_structure_field_class(
+                        members=(("my_int", tc.create_signed_integer_field_class(32)),)
+                    ),
+                    specific_context_field_class=tc.create_structure_field_class(
+                        members=(("my_int", tc.create_signed_integer_field_class(32)),)
+                    ),
+                )
                 test_obj._clock_class = cc
 
         test_obj = self
@@ -172,10 +152,10 @@ class AllMessagesTestCase(unittest.TestCase):
         self._iter = MyIter
 
     def test_all_msg_with_cc(self):
-        params = {'with_cc': True}
-        self._src_comp = self._graph.add_component(self._src, 'my_source', params)
+        params = {"with_cc": True}
+        self._src_comp = self._graph.add_component(self._src, "my_source", params)
         self._msg_iter = TestOutputPortMessageIterator(
-            self._graph, self._src_comp.output_ports['out']
+            self._graph, self._src_comp.output_ports["out"]
         )
 
         for i, msg in enumerate(self._msg_iter):
@@ -206,7 +186,7 @@ class AllMessagesTestCase(unittest.TestCase):
                     type(msg.event.payload_field), bt2_field._StructureFieldConst
                 )
                 self.assertIs(
-                    type(msg.event.payload_field['my_int']),
+                    type(msg.event.payload_field["my_int"]),
                     bt2_field._SignedIntegerFieldConst,
                 )
 
@@ -280,10 +260,10 @@ class AllMessagesTestCase(unittest.TestCase):
                 raise Exception
 
     def test_all_msg_without_cc(self):
-        params = {'with_cc': False}
-        self._src_comp = self._graph.add_component(self._src, 'my_source', params)
+        params = {"with_cc": False}
+        self._src_comp = self._graph.add_component(self._src, "my_source", params)
         self._msg_iter = TestOutputPortMessageIterator(
-            self._graph, self._src_comp.output_ports['out']
+            self._graph, self._src_comp.output_ports["out"]
         )
 
         for i, msg in enumerate(self._msg_iter):
@@ -292,7 +272,7 @@ class AllMessagesTestCase(unittest.TestCase):
                 self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                 self.assertEqual(msg.stream.addr, self._stream.addr)
                 with self.assertRaisesRegex(
-                    ValueError, 'stream class has no default clock class'
+                    ValueError, "stream class has no default clock class"
                 ):
                     msg.default_clock_snapshot
             elif i == 1:
@@ -305,7 +285,7 @@ class AllMessagesTestCase(unittest.TestCase):
                 self.assertIs(type(msg.event.cls), bt2_event_class._EventClassConst)
                 self.assertEqual(msg.event.cls.addr, self._event_class.addr)
                 with self.assertRaisesRegex(
-                    ValueError, 'stream class has no default clock class'
+                    ValueError, "stream class has no default clock class"
                 ):
                     msg.default_clock_snapshot
             elif i == 3:
@@ -317,12 +297,12 @@ class AllMessagesTestCase(unittest.TestCase):
                 self.assertIsNone(msg.stream.cls.default_clock_class)
                 with self.assertRaisesRegex(
                     ValueError,
-                    'such a message has no clock snapshots for this stream class',
+                    "such a message has no clock snapshots for this stream class",
                 ):
                     msg.beginning_default_clock_snapshot
                 with self.assertRaisesRegex(
                     ValueError,
-                    'such a message has no clock snapshots for this stream class',
+                    "such a message has no clock snapshots for this stream class",
                 ):
                     msg.end_default_clock_snapshot
             elif i == 4:
@@ -341,12 +321,12 @@ class AllMessagesTestCase(unittest.TestCase):
                 self.assertIsNone(msg.stream.cls.default_clock_class)
                 with self.assertRaisesRegex(
                     ValueError,
-                    'such a message has no clock snapshots for this stream class',
+                    "such a message has no clock snapshots for this stream class",
                 ):
                     msg.beginning_default_clock_snapshot
                 with self.assertRaisesRegex(
                     ValueError,
-                    'such a message has no clock snapshots for this stream class',
+                    "such a message has no clock snapshots for this stream class",
                 ):
                     msg.end_default_clock_snapshot
             elif i == 6:
@@ -354,18 +334,18 @@ class AllMessagesTestCase(unittest.TestCase):
                 self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                 self.assertEqual(msg.stream.addr, self._stream.addr)
                 with self.assertRaisesRegex(
-                    ValueError, 'stream class has no default clock class'
+                    ValueError, "stream class has no default clock class"
                 ):
                     msg.default_clock_snapshot
             else:
                 raise Exception
 
     def test_msg_stream_with_clock_snapshots(self):
-        params = {'with_cc': True, 'with_stream_msgs_clock_snapshots': True}
+        params = {"with_cc": True, "with_stream_msgs_clock_snapshots": True}
 
-        self._src_comp = self._graph.add_component(self._src, 'my_source', params)
+        self._src_comp = self._graph.add_component(self._src, "my_source", params)
         self._msg_iter = TestOutputPortMessageIterator(
-            self._graph, self._src_comp.output_ports['out']
+            self._graph, self._src_comp.output_ports["out"]
         )
         msgs = list(self._msg_iter)
 
@@ -431,6 +411,23 @@ class CreateDiscardedEventMessageTestCase(unittest.TestCase):
         self.assertIs(type(msg), bt2._DiscardedEventsMessage)
         self.assertEqual(msg.count, 242)
 
+    # With event count == 0.
+    def test_create_with_count_zero_raises(self):
+        def create_stream_class(tc, cc):
+            return tc.create_stream_class(supports_discarded_events=True)
+
+        def msg_iter_next(msg_iter, stream):
+            with self.assertRaisesRegex(
+                ValueError,
+                "discarded event count is 0",
+            ):
+                msg_iter._create_discarded_events_message(stream, count=0)
+
+            return 123
+
+        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
+        self.assertEqual(res, 123)
+
     # With clock snapshots.
     def test_create_with_clock_snapshots(self):
         def create_stream_class(tc, cc):
@@ -457,7 +454,7 @@ class CreateDiscardedEventMessageTestCase(unittest.TestCase):
 
         def msg_iter_next(msg_iter, stream):
             with self.assertRaisesRegex(
-                ValueError, 'stream class does not support discarded events'
+                ValueError, "stream class does not support discarded events"
             ):
                 msg_iter._create_discarded_events_message(stream)
 
@@ -475,7 +472,7 @@ class CreateDiscardedEventMessageTestCase(unittest.TestCase):
         def msg_iter_next(msg_iter, stream):
             with self.assertRaisesRegex(
                 ValueError,
-                'discarded events have no default clock snapshots for this stream class',
+                "discarded events have no default clock snapshots for this stream class",
             ):
                 msg_iter._create_discarded_events_message(
                     stream, beg_clock_snapshot=10, end_clock_snapshot=20
@@ -498,7 +495,7 @@ class CreateDiscardedEventMessageTestCase(unittest.TestCase):
         def msg_iter_next(msg_iter, stream):
             with self.assertRaisesRegex(
                 ValueError,
-                'discarded events have default clock snapshots for this stream class',
+                "discarded events have default clock snapshots for this stream class",
             ):
                 msg_iter._create_discarded_events_message(stream)
 
@@ -507,6 +504,29 @@ class CreateDiscardedEventMessageTestCase(unittest.TestCase):
         res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
         self.assertEqual(res, 123)
 
+    # End clock snapshot greater than beginning clock snapshot.
+    def test_create_clock_snapshots_end_gt_begin_raises(self):
+        def create_stream_class(tc, cc):
+            return tc.create_stream_class(
+                default_clock_class=cc,
+                supports_discarded_events=True,
+                discarded_events_have_default_clock_snapshots=True,
+            )
+
+        def msg_iter_next(msg_iter, stream):
+            with self.assertRaisesRegex(
+                ValueError,
+                r"beginning default clock snapshot value \(20\) is greater than end default clock snapshot value \(10\)",
+            ):
+                msg_iter._create_discarded_events_message(
+                    stream, beg_clock_snapshot=20, end_clock_snapshot=10
+                )
+
+            return 123
+
+        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
+        self.assertEqual(res, 123)
+
 
 class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
     # Most basic case.
@@ -537,6 +557,25 @@ class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
         self.assertIs(type(msg), bt2._DiscardedPacketsMessage)
         self.assertEqual(msg.count, 242)
 
+    # With packet count == 0.
+    def test_create_with_count_zero_raises(self):
+        def create_stream_class(tc, cc):
+            return tc.create_stream_class(
+                supports_packets=True, supports_discarded_packets=True
+            )
+
+        def msg_iter_next(msg_iter, stream):
+            with self.assertRaisesRegex(
+                ValueError,
+                "discarded packet count is 0",
+            ):
+                msg_iter._create_discarded_packets_message(stream, count=0)
+
+            return 123
+
+        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
+        self.assertEqual(res, 123)
+
     # With clock snapshots.
     def test_create_with_clock_snapshots(self):
         def create_stream_class(tc, cc):
@@ -566,7 +605,7 @@ class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
 
         def msg_iter_next(msg_iter, stream):
             with self.assertRaisesRegex(
-                ValueError, 'stream class does not support discarded packets'
+                ValueError, "stream class does not support discarded packets"
             ):
                 msg_iter._create_discarded_packets_message(stream)
 
@@ -586,7 +625,7 @@ class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
         def msg_iter_next(msg_iter, stream):
             with self.assertRaisesRegex(
                 ValueError,
-                'discarded packets have no default clock snapshots for this stream class',
+                "discarded packets have no default clock snapshots for this stream class",
             ):
                 msg_iter._create_discarded_packets_message(
                     stream, beg_clock_snapshot=10, end_clock_snapshot=20
@@ -610,7 +649,7 @@ class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
         def msg_iter_next(msg_iter, stream):
             with self.assertRaisesRegex(
                 ValueError,
-                'discarded packets have default clock snapshots for this stream class',
+                "discarded packets have default clock snapshots for this stream class",
             ):
                 msg_iter._create_discarded_packets_message(stream)
 
@@ -619,6 +658,30 @@ class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
         res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
         self.assertEqual(res, 123)
 
+    # End clock snapshot greater than beginning clock snapshot.
+    def test_create_clock_snapshots_end_gt_begin_raises(self):
+        def create_stream_class(tc, cc):
+            return tc.create_stream_class(
+                default_clock_class=cc,
+                supports_packets=True,
+                supports_discarded_packets=True,
+                discarded_packets_have_default_clock_snapshots=True,
+            )
 
-if __name__ == '__main__':
+        def msg_iter_next(msg_iter, stream):
+            with self.assertRaisesRegex(
+                ValueError,
+                r"beginning default clock snapshot value \(20\) is greater than end default clock snapshot value \(10\)",
+            ):
+                msg_iter._create_discarded_packets_message(
+                    stream, beg_clock_snapshot=20, end_clock_snapshot=10
+                )
+
+            return 123
+
+        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
+        self.assertEqual(res, 123)
+
+
+if __name__ == "__main__":
     unittest.main()
