@@ -1,27 +1,15 @@
+# SPDX-License-Identifier: GPL-2.0-only
 #
 # Copyright (C) 2019 EfficiOS Inc.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; only version 2
-# of the License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-#
 
-import unittest
-import bt2
 import sys
-from utils import TestOutputPortMessageIterator
+import unittest
+
+import bt2
 from bt2 import port as bt2_port
 from bt2 import message_iterator as bt2_message_iterator
+from utils import TestOutputPortMessageIterator
 
 
 class SimpleSink(bt2._UserSinkComponent):
@@ -29,27 +17,27 @@ class SimpleSink(bt2._UserSinkComponent):
     # it.
 
     def __init__(self, config, params, obj):
-        self._add_input_port('in')
+        self._add_input_port("in")
 
     def _user_consume(self):
         next(self._msg_iter)
 
     def _user_graph_is_configured(self):
-        self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+        self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
 
 def _create_graph(src_comp_cls, sink_comp_cls, flt_comp_cls=None):
     graph = bt2.Graph()
 
-    src_comp = graph.add_component(src_comp_cls, 'src')
-    sink_comp = graph.add_component(sink_comp_cls, 'sink')
+    src_comp = graph.add_component(src_comp_cls, "src")
+    sink_comp = graph.add_component(sink_comp_cls, "sink")
 
     if flt_comp_cls is not None:
-        flt_comp = graph.add_component(flt_comp_cls, 'flt')
-        graph.connect_ports(src_comp.output_ports['out'], flt_comp.input_ports['in'])
-        graph.connect_ports(flt_comp.output_ports['out'], sink_comp.input_ports['in'])
+        flt_comp = graph.add_component(flt_comp_cls, "flt")
+        graph.connect_ports(src_comp.output_ports["out"], flt_comp.input_ports["in"])
+        graph.connect_ports(flt_comp.output_ports["out"], sink_comp.input_ports["in"])
     else:
-        graph.connect_ports(src_comp.output_ports['out'], sink_comp.input_ports['in'])
+        graph.connect_ports(src_comp.output_ports["out"], sink_comp.input_ports["in"])
 
     return graph
 
@@ -69,7 +57,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
                 nonlocal the_output_port_from_source
-                the_output_port_from_source = self._add_output_port('out', 'user data')
+                the_output_port_from_source = self._add_output_port("out", "user data")
 
         initialized = False
         graph = _create_graph(MySource, SimpleSink)
@@ -78,7 +66,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
         self.assertEqual(
             the_output_port_from_source.addr, the_output_port_from_iter.addr
         )
-        self.assertEqual(the_output_port_from_iter.user_data, 'user data')
+        self.assertEqual(the_output_port_from_iter.user_data, "user data")
 
     def test_create_from_message_iterator(self):
         class MySourceIter(bt2._UserMessageIterator):
@@ -88,14 +76,14 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MySourceIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         class MyFilterIter(bt2._UserMessageIterator):
             def __init__(self, config, self_port_output):
                 nonlocal flt_iter_initialized
                 flt_iter_initialized = True
                 self._up_iter = self._create_message_iterator(
-                    self._component._input_ports['in']
+                    self._component._input_ports["in"]
                 )
 
             def __next__(self):
@@ -103,8 +91,8 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MyFilter(bt2._UserFilterComponent, message_iterator_class=MyFilterIter):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
-                self._add_output_port('out')
+                self._add_input_port("in")
+                self._add_output_port("out")
 
         src_iter_initialized = False
         flt_iter_initialized = False
@@ -113,6 +101,67 @@ class UserMessageIteratorTestCase(unittest.TestCase):
         self.assertTrue(src_iter_initialized)
         self.assertTrue(flt_iter_initialized)
 
+    # Test that creating a message iterator from a sink component on a
+    # non-connected input port raises.
+    def test_create_from_sink_component_unconnected_port_raises(self):
+        class MySink(bt2._UserSinkComponent):
+            def __init__(comp_self, config, params, obj):
+                comp_self._input_port = comp_self._add_input_port("in")
+
+            def _user_graph_is_configured(comp_self):
+                with self.assertRaisesRegex(ValueError, "input port is not connected"):
+                    comp_self._create_message_iterator(comp_self._input_port)
+
+                nonlocal seen
+                seen = True
+
+            def _user_consume(self):
+                raise bt2.Stop
+
+        seen = False
+        graph = bt2.Graph()
+        graph.add_component(MySink, "snk")
+        graph.run()
+        self.assertTrue(seen)
+
+    # Test that creating a message iterator from a message iterator on a
+    # non-connected input port raises.
+    def test_create_from_message_iterator_unconnected_port_raises(self):
+        class MyFilterIter(bt2._UserMessageIterator):
+            def __init__(iter_self, config, port):
+                input_port = iter_self._component._input_ports["in"]
+
+                with self.assertRaisesRegex(ValueError, "input port is not connected"):
+                    iter_self._create_message_iterator(input_port)
+
+                nonlocal seen
+                seen = True
+
+        class MyFilter(bt2._UserFilterComponent, message_iterator_class=MyFilterIter):
+            def __init__(comp_self, config, params, obj):
+                comp_self._add_input_port("in")
+                comp_self._add_output_port("out")
+
+        class MySink(bt2._UserSinkComponent):
+            def __init__(comp_self, config, params, obj):
+                comp_self._input_port = comp_self._add_input_port("in")
+
+            def _user_graph_is_configured(comp_self):
+                comp_self._input_iter = comp_self._create_message_iterator(
+                    comp_self._input_port
+                )
+
+            def _user_consume(self):
+                raise bt2.Stop
+
+        seen = False
+        graph = bt2.Graph()
+        flt = graph.add_component(MyFilter, "flt")
+        snk = graph.add_component(MySink, "snk")
+        graph.connect_ports(flt.output_ports["out"], snk.input_ports["in"])
+        graph.run()
+        self.assertTrue(seen)
+
     def test_create_user_error(self):
         # This tests both error handling by
         # _UserSinkComponent._create_message_iterator
@@ -120,22 +169,22 @@ class UserMessageIteratorTestCase(unittest.TestCase):
         # are both used in the graph.
         class MySourceIter(bt2._UserMessageIterator):
             def __init__(self, config, self_port_output):
-                raise ValueError('Very bad error')
+                raise ValueError("Very bad error")
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MySourceIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         class MyFilterIter(bt2._UserMessageIterator):
             def __init__(self, config, self_port_output):
                 # This is expected to raise because of the error in
                 # MySourceIter.__init__.
-                self._create_message_iterator(self._component._input_ports['in'])
+                self._create_message_iterator(self._component._input_ports["in"])
 
         class MyFilter(bt2._UserFilterComponent, message_iterator_class=MyFilterIter):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
-                self._add_output_port('out')
+                self._add_input_port("in")
+                self._add_output_port("out")
 
         graph = _create_graph(MySource, SimpleSink, MyFilter)
 
@@ -146,9 +195,9 @@ class UserMessageIteratorTestCase(unittest.TestCase):
         cause = exc[0]
 
         self.assertIsInstance(cause, bt2._MessageIteratorErrorCause)
-        self.assertEqual(cause.component_name, 'src')
-        self.assertEqual(cause.component_output_port_name, 'out')
-        self.assertIn('ValueError: Very bad error', cause.message)
+        self.assertEqual(cause.component_name, "src")
+        self.assertEqual(cause.component_output_port_name, "out")
+        self.assertIn("ValueError: Very bad error", cause.message)
 
     def test_finalize(self):
         class MyIter(bt2._UserMessageIterator):
@@ -158,7 +207,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         finalized = False
         graph = _create_graph(MySource, SimpleSink)
@@ -174,7 +223,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         config_type = None
         graph = _create_graph(MySource, SimpleSink)
@@ -189,14 +238,14 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 nonlocal can_seek_forward
@@ -220,7 +269,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         graph = _create_graph(MySource, SimpleSink)
         with self.assertRaises(bt2._Error) as ctx:
@@ -237,7 +286,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
                 self._salut = 23
 
         salut = None
@@ -257,7 +306,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         called = False
         graph = _create_graph(MySource, SimpleSink)
@@ -272,7 +321,7 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         addr = None
         graph = _create_graph(MySource, SimpleSink)
@@ -308,11 +357,11 @@ class UserMessageIteratorTestCase(unittest.TestCase):
                 tc = self._create_trace_class()
                 sc = tc.create_stream_class(supports_packets=True)
                 ec = sc.create_event_class()
-                self._add_output_port('out', (tc, sc, ec))
+                self._add_output_port("out", (tc, sc, ec))
 
         graph = bt2.Graph()
-        src = graph.add_component(MySource, 'src')
-        it = TestOutputPortMessageIterator(graph, src.output_ports['out'])
+        src = graph.add_component(MySource, "src")
+        it = TestOutputPortMessageIterator(graph, src.output_ports["out"])
 
         # Skip beginning messages.
         msg = next(it)
@@ -331,36 +380,25 @@ class UserMessageIteratorTestCase(unittest.TestCase):
     # This verifies that we are not missing an incref of Py_None, making the
     # refcount of Py_None reach 0.
     def test_try_again_many_times(self):
+        # Starting with Python 3.12, `None` is immortal: its reference
+        # count operations are no-op. Skip this test in that case.
+        before = sys.getrefcount(None)
+        dummy = None  # noqa: F841
+
+        if before == sys.getrefcount(None):
+            raise unittest.SkipTest("`None` is immortal")
+
         class MyIter(bt2._UserMessageIterator):
             def __next__(self):
                 raise bt2.TryAgain
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MyIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
-
-        class MyFilterIter(bt2._UserMessageIterator):
-            def __init__(self, port):
-                input_port = port.user_data
-                self._upstream_iter = self._create_message_iterator(input_port)
-
-            def __next__(self):
-                return next(self._upstream_iter)
-
-            def _user_seek_beginning(self):
-                self._upstream_iter.seek_beginning()
-
-            def _user_can_seek_beginning(self):
-                return self._upstream_iter.can_seek_beginning()
-
-        class MyFilter(bt2._UserFilterComponent, message_iterator_class=MyFilterIter):
-            def __init__(self, config, params, obj):
-                input_port = self._add_input_port('in')
-                self._add_output_port('out', input_port)
+                self._add_output_port("out")
 
         graph = bt2.Graph()
-        src = graph.add_component(MySource, 'src')
-        it = TestOutputPortMessageIterator(graph, src.output_ports['out'])
+        src = graph.add_component(MySource, "src")
+        it = TestOutputPortMessageIterator(graph, src.output_ports["out"])
 
         # Three times the initial ref count of `None` iterations should
         # be enough to catch the bug even if there are small differences
@@ -383,13 +421,13 @@ class UserMessageIteratorTestCase(unittest.TestCase):
 
         class MySource(bt2._UserSourceComponent, message_iterator_class=MySourceIter):
             def __init__(self, config, params, obj):
-                self._add_output_port('out')
+                self._add_output_port("out")
 
         class MyFilterIter(bt2._UserMessageIterator):
             def __init__(self, config, port):
                 # First, create an upstream iterator.
                 self._upstream_iter = self._create_message_iterator(
-                    self._component._input_ports['in']
+                    self._component._input_ports["in"]
                 )
 
                 # Then, voluntarily make a reference cycle that will keep this
@@ -398,16 +436,16 @@ class UserMessageIteratorTestCase(unittest.TestCase):
                 self._self = self
 
                 # Finally, raise an exception to make __init__ fail.
-                raise ValueError('woops')
+                raise ValueError("woops")
 
         class MyFilter(bt2._UserFilterComponent, message_iterator_class=MyFilterIter):
             def __init__(self, config, params, obj):
-                self._in = self._add_input_port('in')
-                self._out = self._add_output_port('out')
+                self._in = self._add_input_port("in")
+                self._out = self._add_output_port("out")
 
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._input_port = self._add_input_port('in')
+                self._input_port = self._add_input_port("in")
 
             def _user_graph_is_configured(self):
                 self._upstream_iter = self._create_message_iterator(self._input_port)
@@ -417,13 +455,13 @@ class UserMessageIteratorTestCase(unittest.TestCase):
                 assert False
 
         g = bt2.Graph()
-        src = g.add_component(MySource, 'src')
-        flt = g.add_component(MyFilter, 'flt')
-        snk = g.add_component(MySink, 'snk')
-        g.connect_ports(src.output_ports['out'], flt.input_ports['in'])
-        g.connect_ports(flt.output_ports['out'], snk.input_ports['in'])
+        src = g.add_component(MySource, "src")
+        flt = g.add_component(MyFilter, "flt")
+        snk = g.add_component(MySink, "snk")
+        g.connect_ports(src.output_ports["out"], flt.input_ports["in"])
+        g.connect_ports(flt.output_ports["out"], snk.input_ports["in"])
 
-        with self.assertRaisesRegex(bt2._Error, 'ValueError: woops'):
+        with self.assertRaisesRegex(bt2._Error, "ValueError: woops"):
             g.run()
 
 
@@ -479,12 +517,12 @@ def _setup_seek_test(
             sc = tc.create_stream_class(supports_packets=True)
             ec = sc.create_event_class()
 
-            self._add_output_port('out', (tc, sc, ec))
+            self._add_output_port("out", (tc, sc, ec))
 
     class MyFilterIter(bt2._UserMessageIterator):
         def __init__(self, config, port):
             self._upstream_iter = self._create_message_iterator(
-                self._component._input_ports['in']
+                self._component._input_ports["in"]
             )
             config.can_seek_forward = self._upstream_iter.can_seek_forward
 
@@ -505,8 +543,8 @@ def _setup_seek_test(
 
     class MyFilter(bt2._UserFilterComponent, message_iterator_class=MyFilterIter):
         def __init__(self, config, params, obj):
-            self._add_input_port('in')
-            self._add_output_port('out')
+            self._add_input_port("in")
+            self._add_output_port("out")
 
     return _create_graph(MySource, sink_cls, flt_comp_cls=MyFilter)
 
@@ -522,10 +560,10 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
     def test_can_seek_beginning(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 nonlocal can_seek_beginning
@@ -556,10 +594,10 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
         # a _user_seek_beginning method.
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 nonlocal can_seek_beginning
@@ -578,10 +616,10 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
         # a _user_seek_beginning method.
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 nonlocal can_seek_beginning
@@ -595,17 +633,17 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
     def test_can_seek_beginning_user_error(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 # This is expected to raise.
                 self._msg_iter.can_seek_beginning()
 
         def _user_can_seek_beginning(self):
-            raise ValueError('moustiquaire')
+            raise ValueError("moustiquaire")
 
         graph = _setup_seek_test(
             MySink,
@@ -617,22 +655,22 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
             graph.run_once()
 
         cause = ctx.exception[0]
-        self.assertIn('ValueError: moustiquaire', cause.message)
+        self.assertIn("ValueError: moustiquaire", cause.message)
 
     def test_can_seek_beginning_wrong_return_value(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 # This is expected to raise.
                 self._msg_iter.can_seek_beginning()
 
         def _user_can_seek_beginning(self):
-            return 'Amqui'
+            return "Amqui"
 
         graph = _setup_seek_test(
             MySink,
@@ -649,10 +687,10 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
     def test_seek_beginning(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 nonlocal do_seek_beginning
@@ -691,16 +729,16 @@ class UserMessageIteratorSeekBeginningTestCase(unittest.TestCase):
     def test_seek_beginning_user_error(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 self._msg_iter.seek_beginning()
 
         def _user_seek_beginning(self):
-            raise ValueError('ouch')
+            raise ValueError("ouch")
 
         graph = _setup_seek_test(MySink, user_seek_beginning=_user_seek_beginning)
 
@@ -932,10 +970,10 @@ class UserMessageIteratorSeekNsFromOriginTestCase(unittest.TestCase):
     ):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 nonlocal can_seek_ns_from_origin
@@ -989,17 +1027,17 @@ class UserMessageIteratorSeekNsFromOriginTestCase(unittest.TestCase):
     def test_can_seek_ns_from_origin_user_error(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 # This is expected to raise.
                 self._msg_iter.can_seek_ns_from_origin(2)
 
         def _user_can_seek_ns_from_origin(self, ns_from_origin):
-            raise ValueError('Joutel')
+            raise ValueError("Joutel")
 
         graph = _setup_seek_test(
             MySink,
@@ -1011,22 +1049,22 @@ class UserMessageIteratorSeekNsFromOriginTestCase(unittest.TestCase):
             graph.run_once()
 
         cause = ctx.exception[0]
-        self.assertIn('ValueError: Joutel', cause.message)
+        self.assertIn("ValueError: Joutel", cause.message)
 
     def test_can_seek_ns_from_origin_wrong_return_value(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 # This is expected to raise.
                 self._msg_iter.can_seek_ns_from_origin(2)
 
         def _user_can_seek_ns_from_origin(self, ns_from_origin):
-            return 'Nitchequon'
+            return "Nitchequon"
 
         graph = _setup_seek_test(
             MySink,
@@ -1043,10 +1081,10 @@ class UserMessageIteratorSeekNsFromOriginTestCase(unittest.TestCase):
     def test_seek_ns_from_origin(self):
         class MySink(bt2._UserSinkComponent):
             def __init__(self, config, params, obj):
-                self._add_input_port('in')
+                self._add_input_port("in")
 
             def _user_graph_is_configured(self):
-                self._msg_iter = self._create_message_iterator(self._input_ports['in'])
+                self._msg_iter = self._create_message_iterator(self._input_ports["in"])
 
             def _user_consume(self):
                 self._msg_iter.seek_ns_from_origin(17)
@@ -1064,5 +1102,5 @@ class UserMessageIteratorSeekNsFromOriginTestCase(unittest.TestCase):
         self.assertEqual(actual_ns_from_origin, 17)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
