@@ -11,7 +11,7 @@ class TheSourceIterator(bt2._UserMessageIterator):
         tc, sc, ec = port.user_data
 
         trace = tc()
-        stream = trace.create_stream(sc, name='the-stream')
+        stream = trace.create_stream(sc, name="the-stream")
 
         self._msgs = [
             self._create_stream_beginning_message(stream),
@@ -31,8 +31,8 @@ class TheSource(bt2._UserSourceComponent, message_iterator_class=TheSourceIterat
     def __init__(self, config, params, obj):
         tc = self._create_trace_class()
         sc = tc.create_stream_class()
-        ec = sc.create_event_class(name='the-event')
-        self._add_output_port('out', user_data=(tc, sc, ec))
+        ec = sc.create_event_class(name="the-event")
+        self._add_output_port("out", user_data=(tc, sc, ec))
 
 
 bt2.register_plugin(__name__, "foo")
